# -*- coding: utf-8 -*-
"""
Created on Sun Mar 20 20:05:08 2022

@author: dcdang
"""

import os
import pandas as pd
import numpy as np
import copy
import random
import seaborn as sns
import pickle
import scipy
from . import source


def BuildTargetMat(bin_st, bin_ed, mat_use, resolution, cut_dist = 5000000):
    """
    Get target Hi-C matrix to calculate domain value. 
    Parameters
    ----------
    bin_st : int
        Index of start bin
    bin_ed : int
        Index of end bin
    mat_use : numpy.array
        Hi-C matrix
    resolution : resolution
        Resolution of Hi-C data
    cut_dist : int, optional
        Range of distance to normalize matrix. The default is 5000000.

    Returns
    -------
    mat_target : numpy.array
        Distance dependent normalized Hi-C matrix between start and end bin. 
        Same shape with mat_use, fill zero in other positions.
    mat_target_norm_z : numpy.array
        Distance dependent normalized Hi-C matrix between start and end bin.
        Part of matrix
    mat_ori : numpy.array
        Original Hi-C matrix between start and end bin.
        Part of matrix
    """

    mat_target = np.zeros([len(mat_use), len(mat_use)])
    mat_use_target = mat_use[bin_st : bin_ed, bin_st:bin_ed] 
    norm_type = 'z-score'
    mat_target_norm_z = source.DistanceNormalizedMatrix(mat_use_target, resolution, norm_type, cut_dist)
    mat_target[bin_st : bin_ed, bin_st : bin_ed] =  mat_target_norm_z
    mat_ori = mat_use[bin_st : bin_ed, bin_st:bin_ed]
    return mat_target, mat_target_norm_z, mat_ori


def GetDomainValue(bin1, bin2, up_bound_use, down_bound_use, mat_target, diag_cut = True):
    """
    Calculate the domain value for selected bin pair.

    Parameters
    ----------
    bin1 : int
        Index of bin1.
    bin2 : int
        Index of bin2.
    up_bound_use : int
        Up bound to calculate up value.
    down_bound_use : int
        Down bound to calculate down value.
    mat_target : numpy.array
        Hi-C matrix used. Need normalization, eg. z-score
    diag_cut : bool, optional
        Whether filter the main diagonal. The default is True.

    Returns
    -------
    domain_value : float
        Mean value within domain.
    up_value : float
        Mean value within up-stream range.
    down_value : float
        Mean value within down-stream range.
    fold_v : float
        Difference between domain value and up or down value.
    pvalue_vec : TYPE
        DESCRIPTION.

    """
    domain_l = bin2 - bin1 + 1
    domain_mat = mat_target[bin1:bin2+1,bin1:bin2+1]
    if diag_cut == True:
        if domain_l <= 5:
            domain_value = np.mean(domain_mat)
            domain_vec = domain_mat.flatten()
        else:
            mat_extract = np.zeros([domain_l, domain_l])
            for j in range(domain_l-1):
                index_l = np.array([1 for i in range(domain_l - j -2)])
                mat_extract += np.diag(index_l, k = j+2)        
            mat_extract += mat_extract.T
            domain_value_all = domain_mat[(mat_extract + mat_extract.T) > 0]
            domain_value = np.mean(domain_value_all)
            domain_vec = domain_value_all
    else:
        domain_value = np.mean(domain_mat)
        domain_vec = domain_mat.flatten()

    #up_range = np.min([bin1 - up_bound, domain_l])
    #down_range = np.min([down_bound - bin2, domain_l])
    up_range = up_bound_use
    down_range = down_bound_use
    ## get up and down-stream regions' value
    if bin1 <= up_range + 1:
        up_value = 0
        up_vec = []
        down_mat = mat_target[bin1:bin2+1, bin2+1:bin2 + down_range + 1]
        down_vec = list(down_mat.flatten())
        down_value = np.mean(down_mat)
    elif bin2 >= len(mat_target) - down_range - 1:
        down_value = 0
        down_vec = []
        up_mat = mat_target[bin1 - up_range:bin1, bin1:bin2+1]
        up_vec = list(up_mat.flatten())
        up_value = np.mean(up_mat)
    else:
        up_mat = mat_target[bin1 - up_range:bin1, bin1:bin2+1]
        down_mat = mat_target[bin1:bin2+1, bin2+1:bin2 + down_range + 1]
        up_vec = list(up_mat.flatten())
        down_vec = list(down_mat.flatten())
        up_value = np.mean(up_mat)
        down_value = np.mean(down_mat)
    if domain_value == 0:
        fold_v = -1
        pvalue_vec = 1
    else:
        bk_vec = np.array(up_vec + down_vec)
        sta_vec, pvalue_vec = scipy.stats.mannwhitneyu(domain_vec, bk_vec, alternative = 'greater')
        #sta_vec, pvalue_vec =  scipy.stats.ttest_ind_from_stats(np.mean(domain_vec), np.std(domain_vec), len(domain_vec), np.mean(bk_vec), np.std(bk_vec), len(bk_vec), equal_var=False)                
                
        #fold_v = (domain_value - np.max([up_value, down_value])) / (domain_value + np.max([up_value, down_value]))
        #fold_v = domain_value - np.max([up_value, down_value])
        if up_value == 0:
            fold_v = domain_value - down_value
        elif down_value == 0:
            fold_v = domain_value - up_value
        else:
            fold_v = domain_value - np.max([up_value, down_value])
        return domain_value, up_value, down_value, fold_v, pvalue_vec


def GetBestBinPair(region_st, score_st, region_ed, score_ed, up_bound_use, down_bound_use, st_cut, mat_target, resolution, weight = 0.5):
    """
    Select the best bin pair in start and end boundary regions.
    Parameters
    ----------
    region_st : list
        Index of bins within start boundary region
    score_st : list
        Boundary score of bins within start boundary region
    region_ed : list
        Index of bins within end boundary region
    score_ed : TYPE
        Boundary score of bins within end boundary region
    up_bound_use : int
        Up bound to calculate up value.
    down_bound_use : int
        Down bound to calculate down value.
    st_cut : int
        Index of bin for end of last bin pair, used for 
        avoiding overlap of domain.
    mat_target : numpy.array
        Hi-C matrix
    resolution : int
        Resolution of Hi-C matrix
    weight : float
        weight to calculate best bin pair. 
        Balance affect of boundary score and domain value. The default is 0.5.

    Returns
    -------
    df_res : pandas.DataFrame
        Record of results for all possible bin pairs between start 
        and end boundary regions.

    """
    df_res = pd.DataFrame(columns = ['region_pair', 'score_pair', 'ave_score', 'domain_v', 'up_v', 'down_v', 'fold_v', 'pvalue_vec'])
    region_il = []
    score_vl = []
    ave_score_vl = []
    domain_vl = []
    up_vl = []
    down_vl = []
    fold_vl = []
    pvec_l = []          
    for i in range(len(region_st)):
        bin1 = region_st[i]
        score1 = score_st[i]
        up_bound_use = up_bound_use + (i + 1)
        if bin1 < st_cut and st_cut != 0:
            continue
        for j in range(len(region_ed)):
            bin2 = region_ed[j]
            score2 = score_ed[j]
            down_bound_use = down_bound_use + (len(region_ed) - j - 1)
            region_il.append([bin1, bin2])
            score_vl.append([score1, score2])
            ave_score_vl.append(np.mean([score1, score2]))
            domain_value, up_value, down_value, fold_v, pvalue_vec = GetDomainValue(bin1, bin2, up_bound_use, down_bound_use, mat_target, diag_cut = True)
            domain_vl.append(domain_value)
            up_vl.append(up_value)
            down_vl.append(down_value)
            fold_vl.append(fold_v)
            pvec_l.append(pvalue_vec)
    df_res['region_pair'] = region_il
    df_res['score_pair'] = score_vl
    df_res['ave_score'] = ave_score_vl
    df_res['domain_v'] = domain_vl
    df_res['up_v'] = up_vl
    df_res['down_v'] = down_vl
    df_res['fold_v'] = fold_vl
    df_res['pvalue_vec'] = pvec_l
    
    score_index = np.argsort(np.array(df_res['ave_score']))
    domain_v_index = np.argsort(np.array(df_res['domain_v']))
    score_rank = np.argsort(score_index)
    domain_v_rank = np.argsort(domain_v_index)
    df_res['score_rank'] = score_rank
    df_res['domain_rank'] = domain_v_rank
    df_res['judge'] = score_rank * weight + (1-weight)*domain_v_rank
    return df_res
    
    
def BoundaryMatch(df_bd_region_type, mat_use, resolution, weight):
    """
    Get best bin pairs between adjacent boundary regions.
    Parameters
    ----------
    df_bd_region_type : pandas.DataFrame
        Dataframe contains boundary region and boundary type.
    mat_use : numpy.array
        DESCRIPTION.
    resolution : int
        Resolution of Hi-C data
    weight : float
        weight to calculate best bin pair. 
        Balance affect of boundary score and domain value. 

    Returns
    -------
    df_record : pandas.DataFrame
        Dataframe to record selected bin pair between adjacent boundary regions

    """
    up_bound_l = list(df_bd_region_type['up_dist'])
    low_bound_l = list(df_bd_region_type['down_dist'])
    record = []
    st_cut = -1
    region_index = []
    for i in range(len(df_bd_region_type)-1):
        region_st = df_bd_region_type['region'][i]
        score_st = df_bd_region_type['score'][i]
        region_ed = df_bd_region_type['region'][i+1]
        score_ed = df_bd_region_type['score'][i+1]        
        up_bound_use = up_bound_l[i]
        down_bound_use = low_bound_l[i]
        
        domain_max = region_ed[-1] - region_st[0] + 1        
        expand_region = np.max([up_bound_use, down_bound_use, domain_max])        
        norm_st = np.max([0, region_st[0] - expand_region])
        norm_ed = np.min([region_ed[-1] + expand_region + 1, len(mat_use)])
        mat_target, mat_target_norm_z, mat_ori = BuildTargetMat(norm_st, norm_ed, mat_use, resolution, cut_dist = 5000000)

        df_res = GetBestBinPair(region_st, score_st, region_ed, score_ed, up_bound_use, down_bound_use, st_cut, mat_target, resolution, weight)
        index_max = np.argmax(np.array(df_res['judge']))       
        region_index.append([i, i+1])
        target = list(df_res.iloc[index_max])
        #if target[0][0] == st_cut:
           #target[0][0] = st_cut + 1 
        record.append(target)
        up_bound_l[i+1] = df_res['region_pair'][index_max][-1] - df_res['region_pair'][index_max][0]
        st_cut = df_res['region_pair'][index_max][-1]
    df_record = pd.DataFrame(record)
    df_record.columns = ['region_pair', 'score_pair', 'ave_score', 'domain_v', 'up_v', 'down_v', 'fold_v', 'pvalue_vec', 'score_rank', 'domain_rank', 'judge']
    df_record['region_record'] = region_index
    return df_record


def GetConsTadBoundary(df_record, df_bd_region_type, Chr, resolution):
    """
    Get ConsTAD domain and boundary
    Parameters
    ----------
    df_record : pandas.DataFrame
        Dataframe to record selected bin pair between adjacent boundary regions
    df_bd_region_type : pandas.DataFrame
        Dataframe contains boundary region and boundary type.
    Chr : str
        Symbol of chromosome
    resolution : int
        Resolution of Hi-C data

    Returns
    -------
    df_tad_cons : pandas.DataFrame
        DataFrame of ConsTADs
    df_boundary_cons : pandas.DataFrame
        DataFrame of ConsTADs boundary

    """
    df_tad_cons = pd.DataFrame(columns = ['chr', 'start', 'end', 'name', 'boundary_st', 'boundary_ed', 'bd_st', 'bd_region_st', 'bd_ed', 'bd_region_ed', 'st_region_type', 'ed_region_type'])
    df_boundary_cons = pd.DataFrame(columns = ['chr', 'start', 'end', 'TAD_name', 'boundary_region', 'region_type'])
    st_l = []
    ed_l = []
    st_use_l = []
    ed_use_l = []
    name_l = []
    bd_st_l = []
    bd_ed_l = []
    region_st_l = []
    region_ed_l = []
    st_type_l = []
    ed_type_l = []
    st_l_bd = []
    ed_l_bd = []
    tad_name_l = []
    region_bd_l = []
    type_bd_l = []
    for i in range(len(df_record)):
        st = df_record['region_pair'][i][0] * resolution
        ed = df_record['region_pair'][i][-1] * resolution + resolution
        st_use = df_record['region_pair'][i][0]
        ed_use = df_record['region_pair'][i][-1]
        name = 'Consensus_' + Chr + '_TAD_' + str(i) 
        bd_st = Chr + ':' + str(st) + '-' + str(st + resolution)
        bd_ed = Chr + ':' + str(ed-resolution) + '-' + str(ed)
        st_index = df_record['region_record'][i][0]
        ed_index = df_record['region_record'][i][-1]
        region_st = df_bd_region_type['region'][st_index]
        region_ed = df_bd_region_type['region'][ed_index]
        st_type = df_bd_region_type['region_type_adjust'][st_index]
        ed_type = df_bd_region_type['region_type_adjust'][ed_index]
        # tad related
        st_l.append(st)
        ed_l.append(ed)
        st_use_l.append(st_use)
        ed_use_l.append(ed_use)
        name_l.append(name)
        bd_st_l.append(bd_st)
        bd_ed_l.append(bd_ed)
        region_st_l.append(region_st)
        region_ed_l.append(region_ed)
        st_type_l.append(st_type)
        ed_type_l.append(ed_type)
        # bd related
        st_l_bd.append(st)
        ed_l_bd.append(st + resolution)
        tad_name_l.append(name)
        region_bd_l.append(region_st)
        type_bd_l.append(st_type)
        
        st_l_bd.append(ed - resolution)
        ed_l_bd.append(ed)
        tad_name_l.append(name)
        region_bd_l.append(region_ed)
        type_bd_l.append(ed_type)
       
    df_tad_cons['start'] = st_l
    df_tad_cons['end'] = ed_l
    df_tad_cons['name'] = name_l
    df_tad_cons['boundary_st'] = bd_st_l
    df_tad_cons['boundary_ed'] = bd_ed_l
    df_tad_cons['bd_region_st'] = region_st_l
    df_tad_cons['bd_st'] = st_use_l
    df_tad_cons['bd_ed'] = ed_use_l
    df_tad_cons['bd_region_ed'] = region_ed_l
    df_tad_cons['st_region_type'] = st_type_l
    df_tad_cons['ed_region_type'] = ed_type_l
    df_tad_cons['chr'] = [Chr for i in range(len(df_tad_cons))]

    df_boundary_cons['start'] = st_l_bd
    df_boundary_cons['end'] = ed_l_bd
    df_boundary_cons['TAD_name'] = tad_name_l
    df_boundary_cons['boundary_region'] = region_bd_l
    df_boundary_cons['region_type'] = type_bd_l
    df_boundary_cons['chr'] = [Chr for i in range(len(df_boundary_cons))]
    return df_tad_cons, df_boundary_cons


def BuildConsTADs(Chr, resolution, mat_use, df_bd_region_type, weight):
    """
    Build ConsTADs from boundary regions.
    Parameters
    ----------
    Chr : str
        Symbol of chromosome
    resolution : int
        Resolution of Hi-C data
    mat_use : numpy.array
        Hi-C matrix
    df_bd_region_type : pandas.DataFrame
        
    weight : float
        weight to calculate best bin pair. 
        Balance affect of boundary score and domain value.  

    Returns
    -------
    df_tad_cons : pandas.DataFrame
        DataFrame of ConsTADs
    df_boundary_cons : pandas.DataFrame
        DataFrame of ConsTADs boundary

    """
    df_record = BoundaryMatch(df_bd_region_type, mat_use, resolution, weight)
    df_tad_cons, df_boundary_cons = GetConsTadBoundary(df_record, df_bd_region_type, Chr, resolution)
    print('Get ' + str(len(df_tad_cons)) + ' ConsTADs')
    return df_tad_cons, df_boundary_cons










#!/usr/bin/env python
"""
# Author: Dachang Dang
# File Name: __init__.py
# Description:
"""

__author__ = "Dachang Dnag"
__email__ = "dangdachang@163.com"

__all__ = ['ConsTADs_main', 'GetConsTADs', 'IdentifyBoundaryRegions', 'Preprocess', 'PlotFunction',
           'TadSeparationLandscape', 'source']







# -*- coding: utf-8 -*-
"""
Created on Fri Mar 18 21:39:46 2022

@author: dcdang
"""

import os
import pandas as pd
import numpy as np
import time
import scipy
import scipy.sparse
import scipy.stats 
import copy
import random
from . import source


def __get_bin_name_list_for_chr(chr_length, chr_symbol, resolution):
     start_pos = 0
     start = []
     end = []
     bin_name_chr = []
     while (start_pos + resolution) <= chr_length:
          start.append(start_pos)
          end.append(start_pos + resolution)
          start_pos += resolution
     start.append(start_pos)
     end.append(chr_length)
     for i in range(len(start)):
          bin_name_chr.append(chr_symbol + ':' + str(start[i]) + '-' + str(end[i]))
     return bin_name_chr


def __CollectBdScore(TAD_result, method_list, bin_name_chr, expand_bin = 0):
    """
    Collect scores contributed by each TAD caller.

    Parameters
    ----------
    TAD_result : dict
        Dictionary contains the the uniform format of TAD domain and TAD boundary
    method_list : list
        List of TAD caller
    bin_num : int
        Number of bins along the chromosome
    expand_bin : int, optional
        Window size of boundary voting. The default is 0.

    Returns
    -------
    boundary_score_record : dict
        Dictionary contains the boundary scores for each bin contributed by each TAD caller

    """
    boundary_score_record = {}
    for method in method_list:
        boundary_score_record[method] = {}
        df_domain = TAD_result[method]['TAD_domain']
        method_start_bin_score = np.zeros(len(bin_name_chr))
        method_end_bin_score = np.zeros(len(bin_name_chr))       
        bd_st_record = []
        bd_ed_record = []
        for i in range(len(df_domain)):
            boundary_st = df_domain['boundary_st'][i]
            boundary_ed = df_domain['boundary_ed'][i]
            if boundary_st not in bd_st_record:
                st_ind = bin_name_chr.index(boundary_st)
                method_start_bin_score[st_ind - expand_bin : st_ind + expand_bin + 1] += 1
                bd_st_record.append(boundary_st)
            if boundary_ed not in bd_ed_record:
                ed_ind = bin_name_chr.index(boundary_ed)
                method_end_bin_score[ed_ind - expand_bin : ed_ind + expand_bin + 1] += 1
                bd_ed_record.append(boundary_ed)
        if np.max(method_start_bin_score) > 1 or np.max(method_end_bin_score) > 1:
            print('Wrong bd score contribute!')
        boundary_score_record[method]['start_bin_score'] = method_start_bin_score
        boundary_score_record[method]['end_bin_score'] = method_end_bin_score
    return boundary_score_record


def __GetBdScore(boundary_score_record, method_list):
    """
    Get the boundary scores for each bin along the chromosome 
    based on the score contributed by all TAD callers 

    Parameters
    ----------
    boundary_score_record : dict
        Dictionary contains the boundary scores for each bin contributed by each TAD caller
    method_list : list
        List of TAD caller

    Returns
    -------
    bd_score_final : pandas.DataFrame
        Boundary scores for each bin along the chromosome.

    """
    df_method_bd_score_st_ed = pd.DataFrame(columns = method_list)
    for method in method_list:
        bd_score_st_ed = []
        bd_score_method_st = boundary_score_record[method]['start_bin_score']
        bd_score_method_ed = boundary_score_record[method]['end_bin_score']
        bd_score_st_ed.append(list(bd_score_method_st))
        bd_score_st_ed.append(list(bd_score_method_ed))
        bd_score_st_ed_max = np.max(bd_score_st_ed, axis = 0)
        df_method_bd_score_st_ed[method] = bd_score_st_ed_max
    bd_score_final = pd.DataFrame(np.sum(df_method_bd_score_st_ed, axis = 1))
    bd_score_final.columns = ['bd_score']
    return bd_score_final


def BoundaryVoting(TAD_result, method_list, bin_num, chr_length, chr_symbol, resolution, expand_bin = 0):
    """
    Process of boundary voting among different TAD callers.

    Parameters
    ----------
    TAD_result : dict
        Dictionary contains the the uniform format of TAD domain and TAD boundary
    method_list : list
        List of TAD caller
    bin_num : int
        Number of bins along the chromosome
    expand_bin : int, optional
        Window size of boundary voting. The default is 0.

    Returns
    -------
     bd_score_final : pandas.DataFrame
        Boundary scores for each bin along the chromosome.

    """
    bin_name_chr = __get_bin_name_list_for_chr(chr_length, chr_symbol, resolution)
    boundary_score_record = __CollectBdScore(TAD_result, method_list, bin_name_chr, expand_bin)
    bd_score_primary = __GetBdScore(boundary_score_record, method_list)
    return bd_score_primary



def GetMultiScaleContrastPvalue(bd_score_primary, mat_use, window_list, resolution):    
    """
    Calculate the multi-scale contrast P-value according to the input window_list.

    Parameters
    ----------
    bd_score_final : pandas.DataFrame
        Primary boundary score for each bin along the chromosome.
    mat_use : numpy.array
        Hi-C matrix used for contrast P-value calculation, need normalized (eg. z-score).
    window_list : list
        List of window size for multi-scale contrast P-value, in base pair.
    resolution : int
        Hi-C matrix resolution.

    Returns
    -------
    df_pvalue_result : pandas.DataFrame
    DataFrame contains the multi-scale contrast P-value for each bin along the chromosome    

    """
    df_pvalue_result = pd.DataFrame()
    for square_size in window_list:
        scale_label = str(int(square_size / 1000)) + 'kb-window'
        #print('Dealing with ' + scale_label)
        square_size = int(square_size / resolution)
        Sta_value_list = []
        mat_extract = np.zeros([square_size, square_size])
        for j in range(square_size):
            index_l = np.array([1 for i in range(square_size - j -1)])
            mat_extract += np.diag(index_l, k = j+1)        
        for i in range(len(mat_use)):
            if (i <= (square_size-1)) or (i >= (len(mat_use)-square_size)):
                Sta_value_list.append(-1)
            else:
                #y1 = x.ravel() modify can change original matrix
                #y2 = x.flatten() modify not change original matrix
                Cross_value = mat_use[i - square_size:i, i+1:i+square_size+1].flatten()
                up_mat = np.triu(mat_use[i - square_size:i, i - square_size:i], k = 1)
                down_mat = np.tril(mat_use[i+1:i+square_size+1, i+1:i+square_size+1], k =-1)                                
                #diag_ave_value = (np.diag(mat_use)[i - square_size:i] + np.diag(mat_use)[i+1:i+square_size+1]) /2
                #diag_ave_mat = np.diag(diag_ave_value)                
                #intra_mat = up_mat + down_mat + diag_ave_mat
                #Intra_value = intra_mat.flatten()
                
                intra_mat = up_mat + down_mat
                Intra_value = intra_mat[(mat_extract + mat_extract.T) > 0]
                if np.sum(Cross_value==0) >= len(Cross_value) or np.sum(Intra_value==0) >= len(Intra_value):
                    Sta_value_list.append(1)
                    continue
                sta, pvalue = scipy.stats.mannwhitneyu(Cross_value, Intra_value, alternative = 'less')
                #sta, pvalue =  scipy.stats.ttest_ind_from_stats(np.mean(Cross_value), np.std(Cross_value), len(Cross_value), np.mean(Intra_value), np.std(Intra_value), len(Intra_value), equal_var=False)                
                Sta_value_list.append(pvalue)
        df_pvalue_result[scale_label] = Sta_value_list
    df_pvalue_result['bd_score'] = bd_score_primary['bd_score']
    return df_pvalue_result


def GetMultiScaleContrastIndex(bd_score_primary, mat_use, window_list, resolution):    
    
    """
    Calculate the multi-scale contrast index according to the input window_list.

    Parameters
    ----------
    bd_score_primary : pandas.DataFrame
        Primary boundary score for each bin along the chromosome.
    mat_use : numpy.array
        Hi-C matrix used for contrast index calculation, need normalization (eg. z-score).
    window_list : list
        List of window size for multi-scale contrast index, in base pair.
    resolution : int
        Hi-C matrix resolution.

    Returns
    -------
    df_pvalue_result : pandas.DataFrame
    DataFrame contains the multi-scale contrast P-value for each bin along the chromosome    

    """
    df_Insvalue_result = pd.DataFrame()
    for square_size in window_list:
        scale_label = str(int(square_size / 1000)) + 'kb-window'
        print('Dealing with ' + scale_label)
        square_size = int(square_size / resolution)
        Ins_value_list = []
        mat_extract = np.zeros([square_size, square_size])
        for j in range(square_size):
            index_l = np.array([1 for i in range(square_size - j -1)])
            mat_extract += np.diag(index_l, k = j+1)        
        for i in range(len(mat_use)):
            if (i <= (square_size-1)) or (i >= (len(mat_use)-square_size)):
                Ins_value_list.append(-1)
            else:                
                #y1 = x.ravel() modify can change original matrix
                #y2 = x.flatten() modify not change original matrix
                Cross_value = mat_use[i - square_size:i, i+1:i+square_size+1].flatten()
                up_mat = np.triu(mat_use[i - square_size:i, i - square_size:i], k = 1)
                down_mat = np.tril(mat_use[i+1:i+square_size+1, i+1:i+square_size+1], k =-1)                                
                #diag_ave_value = (np.diag(mat_use)[i - square_size:i] + np.diag(mat_dense)[i+1:i+square_size+1]) /2
                #diag_ave_mat = np.diag(diag_ave_value)                
                #intra_mat = up_mat + down_mat + diag_ave_mat
                #Intra_value = intra_mat.flatten()
                
                intra_mat = up_mat + down_mat
                Intra_value = intra_mat[(mat_extract + mat_extract.T) > 0]
                if np.mean(Intra_value) == 0:
                    Ins_value_list.append(0)
                else:
                    CI_value = (np.mean(Intra_value) - np.mean(Cross_value)) / (np.mean(Intra_value) + np.mean(Cross_value))
                    Ins_value_list.append(CI_value)
        df_Insvalue_result[scale_label] = Ins_value_list
    df_Insvalue_result['bd_score'] = bd_score_primary['bd_score']
    return df_Insvalue_result


def GetBestWindowSizeForContrastPvalue(bd_score_primary, mat_use, window_list_multi, resolution):    
    """
    Select the best window size of the contrast P-value.

    Parameters
    ----------
    bd_score_primary : pandas.DataFrame
        Primary boundary score for each bin along the chromosome.
    mat_use : numpy.array
        Hi-C matrix used for contrast index calculation, need normalization (eg. z-score).
    window_list : list
        List of window size for multi-scale contrast index, in base pair.
    resolution : int
        Hi-C matrix resolution.

    Returns
    -------
    w_best : str
        Best window size for contrast P-value.
    df_pvalue_score_cor : pandas.DataFrame
        DataFrame contains pearson correlation of boundary score profile 
        and contrast P-value with different window size .

    """
    df_pvalue_score_cor = pd.DataFrame()
    w_list_multi = []
    cor_result_cell = []
    for w_use in window_list_multi:
       w_list_multi.append(str(int(w_use / 1000)) + 'kb-window')
    df_bd_insul_pvalue_multi =  GetMultiScaleContrastPvalue(bd_score_primary, mat_use, window_list_multi, resolution)
    for i in range(len(w_list_multi)):                
        w_use = w_list_multi[i]
        window_size = window_list_multi[i]
        w_cut = int(window_size / resolution)                
        cor_pvalue_score = scipy.stats.pearsonr(np.array(bd_score_primary['bd_score'].iloc[w_cut:len(bd_score_primary)-w_cut+1]), -np.array(df_bd_insul_pvalue_multi[w_use].iloc[w_cut:len(bd_score_primary)-w_cut+1]))[0]
        #cor_pvalue_score = scipy.stats.pearsonr(np.array(bd_score_primary['bd_score']), -np.array(df_bd_insul_pvalue_multi[w_use]))[0]
        #print('For ' + str(w_use))
        #print(cor_pvalue_score)
        cor_result_cell.append(cor_pvalue_score)            
    w_best = w_list_multi[np.argmax(cor_result_cell)]
    print('Best window size:' + w_best)
    df_pvalue_score_cor = pd.DataFrame(cor_result_cell)
    df_pvalue_score_cor.columns = ['PCC']
    df_pvalue_score_cor['window'] = w_list_multi
    return w_best, df_bd_insul_pvalue_multi, df_pvalue_score_cor


def GetBoundaryRegion(bd_score_primary, Chr):
    """
    Get boundary region from boundary score profile.

    Parameters
    ----------
    bd_score_primary : pandas.DataFrame.
        Primary boundary score for each bin along the chromosome.
    Chr : str
        Symbol of chromosome.

    Returns
    -------
    df_boundary_region : pandas.DataFrame.
        DataFrame contains the boundary region defined from the boundary score profile.

    """
    df_boundary_region = pd.DataFrame(columns = ['chr', 'start', 'end', 'length', 'region', 'score', 'ave_score', 'max_score', 'up_dist', 'down_dist'])
    st_list = []
    ed_list = []
    length_list = []
    region_l = []
    region_score_l = []
    ave_score_l = []
    max_score_l = []
    up_d_l = []
    down_d_l = []
    tad_seperation_score_new = copy.deepcopy(bd_score_primary)
    region_on = False
    for i in range(len(tad_seperation_score_new)):
        bd_score = tad_seperation_score_new['bd_score'][i]
        if bd_score != 0 and region_on == False:
            target = []
            score_r = []
            target.append(i)
            score_r.append(bd_score)
            region_on = True
            continue
        elif bd_score != 0 and region_on == True:
            target.append(i)
            score_r.append(bd_score)
        elif bd_score == 0 and region_on == True:
            region_on = False
            st_list.append(target[0])
            ed_list.append(target[-1])
            length_list.append(len(target))
            region_l.append(target)
            region_score_l.append(score_r)
            ave_score_l.append(np.mean(score_r))                    
            max_score_l.append(np.max(score_r))
            continue
        else:
            continue
    if region_on == True:
        st_list.append(target[0])
        ed_list.append(target[-1])
        length_list.append(len(target))
        region_l.append(target)
        region_score_l.append(score_r)
        ave_score_l.append(np.mean(score_r))                    
        max_score_l.append(np.max(score_r))
        region_on = False      
    df_boundary_region['start'] = st_list
    df_boundary_region['end'] = ed_list
    df_boundary_region['length'] = length_list
    df_boundary_region['region'] = region_l
    df_boundary_region['score'] = region_score_l
    df_boundary_region['ave_score'] = ave_score_l
    df_boundary_region['max_score'] = max_score_l                        
    df_boundary_region['chr'] = [Chr for j in range(len(df_boundary_region))]
    for i in range(len(df_boundary_region)):
        if i == 0:
            up_d_l.append(0)
            down_d_l.append(df_boundary_region['start'][i+1] - df_boundary_region['end'][i])
        elif i == len(df_boundary_region) - 1:
            down_d_l.append(0)
            up_d_l.append(df_boundary_region['start'][i] - df_boundary_region['end'][i-1])
        else:
            up_d_l.append(df_boundary_region['start'][i] - df_boundary_region['end'][i-1])            
            down_d_l.append(df_boundary_region['start'][i+1] - df_boundary_region['end'][i])
    df_boundary_region['up_dist'] = up_d_l
    df_boundary_region['down_dist'] = down_d_l
    return df_boundary_region


def AddOperation(bd_score_cell, df_bd_insul_pvalue, Chr, w_best, p_cut):
    """
    Add operation to refine the boundary score profile 
    by add 1 score to bins with good contrast P-value.

    Parameters
    ----------
    bd_score_cell : pandas.DataFrame
        boundary score for each bin along the chromosome.  
    df_bd_insul_pvalue : pandas.DataFrame
    DataFrame contains the multi-scale contrast P-value for each bin along the chromosome
    Chr : str
        Symbol of chromosome.
    w_best : str
        Best window size for contrast P-value.
    p_cut : float
        cut-off of contrast P-value

    Returns
    -------
    df_boundary_region_add : pandas.DataFrame.
        DataFrame contains the boundary region defined from the boundary score profile.
    bd_score_cell_add : pandas.DataFrame.
        boundary score for each bin along the chromosome after Add operation

    """
    num = 0
    bd_score_cell_add = copy.deepcopy(bd_score_cell)
    for i in range(len(bd_score_cell)):
        bd_score = bd_score_cell['bd_score'][i]
        bd_pvalue = df_bd_insul_pvalue[w_best][i]
        if bd_pvalue == -1:
            continue
        if bd_score == 0 and bd_pvalue <= p_cut:
            bd_score_cell_add['bd_score'][i] = 1
            num += 1
    print('Add score for ' + str(num) + ' bins')
    df_boundary_region_add = GetBoundaryRegion(bd_score_cell_add, Chr)
    return df_boundary_region_add, bd_score_cell_add
    

def GetLocalMinInPvalue(df_bd_insul_pvalue, w_best, p_cut):   
    local_min_judge = [0]
    for i in range(1, len(df_bd_insul_pvalue)-1):
        if df_bd_insul_pvalue[w_best][i] == -1 or df_bd_insul_pvalue[w_best][i] <= p_cut:
            local_min_judge.append(0)
        else:
            p_h = df_bd_insul_pvalue[w_best][i]
            p_up = df_bd_insul_pvalue[w_best][i-1]
            p_down = df_bd_insul_pvalue[w_best][i+1]
            p_dif = np.max([p_up - p_h, p_down - p_h])
            if p_h <= p_up and p_h <= p_down and p_dif > 0.01 and p_h < 0.9:
                local_min_judge.append(1)
            else:
                local_min_judge.append(0)
    local_min_judge.append(0)
    expand_l = []
    for i in range(2, len(local_min_judge)-2):
        if local_min_judge[i] == 0:
            continue
        else:
            up2 = df_bd_insul_pvalue[w_best][i-2]
            up1 = df_bd_insul_pvalue[w_best][i-1]
            down1 = df_bd_insul_pvalue[w_best][i+1]
            down2 = df_bd_insul_pvalue[w_best][i+2]
            if up1 < up2 and up1 < down2:
                expand_l.append(i-1)
            if down1 < up2 and down1 < down2:
                expand_l.append(i+1)
    for i in range(len(expand_l)):
        local_min_judge[expand_l[i]] = 1
    return local_min_judge


def FilterOperation(bd_score_cell, df_bd_insul_pvalue, Chr, w_best, p_cut, high_score_cut = 5):
    """
    Filter operation to refine the boundary score profile 
    by filter out bins with bad contrast P-values.

    Parameters
    ----------
    bd_score_cell : pandas.DataFrame
        boundary score for each bin along the chromosome.  
    df_bd_insul_pvalue : pandas.DataFrame
     DataFrame contains the multi-scale contrast P-value for each bin along the chromosome
    Chr : str
        Symbol of chromosome.
    w_best : str
        Best window size for contrast P-value.
    p_cut : float
        cut-off of contrast P-value
    high_score_cut : int, optional
        Cut off boundary score. The default is 5.
        Bins with score abover the cut off will not be filtered.

    Returns
    -------
    df_boundary_region_adjust : pandas.DataFrame.
        DataFrame contains the boundary region defined from the boundary score profile.
    bd_score_cell_adjust : TYPE
        boundary score for each bin along the chromosome after Filter operation

    """
    local_min_judge = GetLocalMinInPvalue(df_bd_insul_pvalue, w_best, p_cut)      
    bd_score_cell_adjust = copy.deepcopy(bd_score_cell)
    bd_score_cell_adjust[(bd_score_cell_adjust['bd_score'] < high_score_cut) & (df_bd_insul_pvalue[w_best] > p_cut)] = 0
    for i in range(len(local_min_judge)):
        if local_min_judge[i] == 0:
            continue
        else:
            score_hold = copy.deepcopy(bd_score_cell['bd_score'][i])
            if bd_score_cell_adjust['bd_score'][i] == 0:
                bd_score_cell_adjust['bd_score'][i] = score_hold
    df_boundary_region_adjust = GetBoundaryRegion(bd_score_cell_adjust, Chr)
    return df_boundary_region_adjust, bd_score_cell_adjust
 

def CombineOperation(df_boundary_region, bd_score_cell, Chr, combine_dist = 2):
    """
    Combine operation to refine the boundary score profile 
    by combine two close boundary regions.

    Parameters
    ----------
    df_boundary_region : pandas.DataFrame.
        DataFrame contains the boundary region defined from the boundary score profile.
    bd_score_cell : pandas.DataFrame
        boundary score for each bin along the chromosome.
    Chr : str
        Symbol of chromosome.
    combine_dist : int, optional
        Distance cut off to combine two boundary regions. The default is 2.

    Returns
    -------
    df_boundary_region_combine : pandas.DataFrame.
        DataFrame contains the boundary region defined from the boundary score profile.
    bd_score_cell_combine : pandas.DataFrame.
        boundary score for each bin along the chromosome after Add operation
    """
    bd_score_cell_combine = copy.deepcopy(bd_score_cell)
    num = 0
    for i in range(len(df_boundary_region)-1):
        ed = df_boundary_region['end'][i]
        down_dist = df_boundary_region['down_dist'][i]
        if down_dist <= combine_dist:
            st_next = df_boundary_region['start'][i+1]
            fill_score = (bd_score_cell['bd_score'][ed] + bd_score_cell['bd_score'][st_next]) / 2
            bd_score_cell_combine[ed + 1: ed + down_dist] = fill_score
            num += 1
    #print('There are ' + str(num) + ' times combination.')
    df_boundary_region_combine = GetBoundaryRegion(bd_score_cell_combine, Chr)
    return df_boundary_region_combine, bd_score_cell_combine 


def BuildTadSeperationLandscape(Chr, resolution, mat_file, bin_num, mat_type, mat_norm_check, bd_score_cell, window_list, 
                                   p_cut, high_score_cut, combine_dist, norm_type = 'z-score', cut_dist = 12000000):
    """
    Function to build TAD separation landscape.

    Parameters
    ----------
    Chr : str
        Symbol of chromosome
    resolution : int
        Resolution of Hi-C data
    mat_file : str
        File path of Hi-C matrix
    bin_num : int
        Number of bins for target chromosome
    mat_type : str
        Type of Hi-C matrix, dense or sparse
    mat_norm_check : bool
        For distance-dependent normalized Hi-C matrix: True;
        otherwise: False
    bd_score_cell : pandas.DataFrame
        boundary score for each bin along the chromosome.
    window_list : list
        Multi-scale window size for contrast P-value computation (base pair)
    p_cut : float
        Cut-off for contrast P-value
    high_score_cut : int
        Cut-off for boundary score filter.
    combine_dist : int
        Cut-off for combination of nearby boundary regions
    norm_type : str, optional
        Type of distance-dependent normalization. The default is 'z-score', 
        other like 'min-max', 'obs_exp'. 
    cut_dist : int, optional
        Up-bound for distance-dependent normalization. The default is 12000000.

    Returns
    -------
    mat_dense : numpy.array
        Dense Hi-C matrix
    mat_norm : numpy.array
        Distance-dependent normalized Hi-C matrix
    result_record : dict
        Boundary score profile and correspondig boundary regions 
        obtained by three operations
    w_best : str
        Best window for contrast P-value
    df_bd_insul_pvalue : pandas.DataFrame
        Multi-scale contrast P-value for bins along the chromosome
    df_pvalue_score_cor : TYPE
        Pearson correlation between Multi-scale contrast P-value and primary boundary score profile

    """
    result_record = {}
    print('Load Hi-C matrix...')
    mat_dense = source.LoadHicMat(mat_file, bin_num, mat_type)
    if mat_norm_check == False:
        print('Normalize the Hi-C matrix...')
        mat_norm = source.DistanceNormalizedMatrix(mat_dense, resolution, norm_type, cut_dist)
    else:
        print('Normalization done')
        mat_norm = mat_dense
    
    print('Calculate multi-scale contrast pvalue...')    
    w_best, df_bd_insul_pvalue, df_pvalue_score_cor = GetBestWindowSizeForContrastPvalue(bd_score_cell, mat_norm, window_list, resolution)
    
    print('Building TAD seperation landscape...')
    df_boundary_region = GetBoundaryRegion(bd_score_cell, Chr)    
    result_record['Original'] = {}
    result_record['Original']['bd_region'] = df_boundary_region
    result_record['Original']['TAD_score'] = bd_score_cell
    
    print('Operation 1: Add')
    df_boundary_region_add, bd_score_cell_add = AddOperation(bd_score_cell, df_bd_insul_pvalue, Chr, w_best, p_cut)
    result_record['Add'] = {}
    result_record['Add']['bd_region'] = df_boundary_region_add
    result_record['Add']['TAD_score'] = bd_score_cell_add
    
    print('Operation 2: Filter')
    df_boundary_region_adjust, bd_score_cell_adjust = FilterOperation(bd_score_cell_add, df_bd_insul_pvalue, Chr, w_best, p_cut, high_score_cut)
    result_record['Filter'] = {}
    result_record['Filter']['bd_region'] = df_boundary_region_adjust
    result_record['Filter']['TAD_score'] = bd_score_cell_adjust
       
    print('Operation 3: Combine')
    df_boundary_region_combine, bd_score_cell_combine = CombineOperation(df_boundary_region_adjust, bd_score_cell_adjust, Chr, combine_dist)
    result_record['Combine'] = {}
    result_record['Combine']['bd_region'] = df_boundary_region_combine
    result_record['Combine']['TAD_score'] = bd_score_cell_combine
    
    return mat_dense, mat_norm, result_record, w_best, df_bd_insul_pvalue, df_pvalue_score_cor




































# -*- coding: utf-8 -*-
"""
Created on Sun Mar 20 15:01:56 2022

@author: dcdang
"""

import os
import pandas as pd
import numpy as np
import scipy
import random
import seaborn as sns
from sklearn import preprocessing
from sklearn.cluster import KMeans
from sklearn.metrics import silhouette_score
import matplotlib.pyplot as plt


def DrawKmeansSseWithMaxCnum(df_boundary_region_combine, K, save_name = ''):
    """
    Draw the sum of square error of K-means with maximal cluster number
    
    Parameters
    ----------
    df_boundary_region_combine : pandas.DataFrame
        DataFrame contains the boundary region defined from the boundary score profile.
    K : int
        Maximal cluster number
    save_name : TYPE, optional
        save path and name of figure. The default is ''.

    Returns
    -------
    None.

    """
    X = np.array(df_boundary_region_combine[['length', 'ave_score']])
    T = preprocessing.StandardScaler().fit(X)
    X=T.transform(X)
    SSE = []
    for k in range(1, K):
        estimator = KMeans(n_clusters=k)
        estimator.fit(X)
        SSE.append(estimator.inertia_)
    plt.figure(figsize=(5.5, 4.2))
    plt.xlabel('Cluster Number', FontSize = 12)
    plt.ylabel('SSE', FontSize = 12)
    plt.plot(range(1, K), SSE, 'o-', linewidth = 2)
    plt.xticks(FontSize = 12)
    plt.yticks(FontSize = 12)
    ax=plt.gca()
    ax.spines['bottom'].set_linewidth(1.6)
    ax.spines['left'].set_linewidth(1.6)
    ax.spines['right'].set_linewidth(1.6)
    ax.spines['top'].set_linewidth(1.6)
    ax.tick_params(axis = 'y', length=7, width = 1.6)
    ax.tick_params(axis = 'x', length=3, width = 1.6)    
    if save_name != '':
        plt.savefig(save_name, format = 'svg', transparent = True) 
    plt.show()
    fig = plt.gcf() 
    plt.close(fig)
    

def DrawKmeansSilhouetteScoreWithMaxCnum(df_boundary_region_combine, K, save_name = ''):
    """
    Draw the silhouette score of K-means with maximal cluster number
    
    Parameters
    ----------
    df_boundary_region_combine : pandas.DataFrame
        DataFrame contains the boundary region defined from the boundary score profile.
    K : int
        Maximal cluster number
    save_name : TYPE, optional
        save path and name of figure. The default is ''.

    Returns
    -------
    None.

    """
    X = np.array(df_boundary_region_combine[['length', 'ave_score']])
    T = preprocessing.StandardScaler().fit(X)
    X=T.transform(X)
    Scores = [] 
    for k in range(2, K):
        estimator = KMeans(n_clusters=k)
        estimator.fit(X)
        Scores.append(silhouette_score(X, estimator.labels_, metric='euclidean'))
    plt.figure(figsize=(6, 4))
    plt.xlabel('Cluster Number', FontSize = 12)
    plt.ylabel('Silhouette score', FontSize = 12)
    plt.plot(range(2, K), Scores, 'o-', linewidth = 2)
    plt.xticks(FontSize = 12)
    plt.yticks(FontSize = 12)
    ax=plt.gca()
    ax.spines['bottom'].set_linewidth(1.6)
    ax.spines['left'].set_linewidth(1.6)
    ax.spines['right'].set_linewidth(1.6)
    ax.spines['top'].set_linewidth(1.6)
    ax.tick_params(axis = 'y', length=7, width = 1.6)
    ax.tick_params(axis = 'x', length=3, width = 1.6)    
    if save_name != '':
        plt.savefig(save_name, format = 'svg', transparent = True) 
    plt.show()
    fig = plt.gcf() #获取当前figure
    plt.close(fig)


def GetBoundaryRegionsWithKmeans(df_boundary_region_combine, K, color_use, save_name = '', permute=True):
    """
    Identify three types of boundary regions with Kmeans 
    based on the length and average scores of boundary regions 

    Parameters
    ----------
    df_boundary_region_combine : pandas.DataFrame
        DataFrame contains the boundary region defined from the boundary score profile.
    K : int
        Number of clusters (Types of boundary regions).
    color_use : list
        Color used for drawing scatter plot of boundary regions.
    save_name : str, optional
        save path and name of figure. The default is ''.
    permute : bool, optional
        Whether permute scatters to avoid overlap. The default is True.

    Returns
    -------
    df_boundary_region_combine : pandas.DataFrame
        DataFrame contains the boundary region with types.

    """
    X = np.array(df_boundary_region_combine[['length', 'ave_score']])
    T = preprocessing.StandardScaler().fit(X)
    Xn=T.transform(X)
    estimator = KMeans(n_clusters=K)
    estimator.fit(Xn)
    label_pred = estimator.labels_
    df_boundary_region_combine['region_label'] = list(label_pred)
    length_r_l = []
    score_r_l = []
    for i in range(K):
        x = X[label_pred == i]
        #print(len(x))
        length_r_l.append(np.mean(x[:, 0]))
        score_r_l.append(np.mean(x[:, 1]))    
    ind_len_max = np.argmax(length_r_l)
    ind_score_max = np.argmax(score_r_l)
    region_type = {}
    region_type[ind_len_max] = 'Wide'
    region_type[ind_score_max] = 'Narrow-strong'
    for k in range(K):
        if k == ind_len_max or k == ind_score_max:
            continue
        else:
            region_type[k] = 'Narrow-weak'
    #print(region_type)
    region_type_l = []
    for i in range(len(label_pred)):
        region_type_l.append(region_type[label_pred[i]])
    df_boundary_region_combine['region_type'] = list(region_type_l)
    
    '''
    plt.figure(figsize=(5, 4.2))
    type_list_r = ['Narrow-strong', 'Narrow-weak', 'Wide']
    for i in range(len(type_list_r)) :
        type_r = type_list_r[i]
        x = X[df_boundary_region_combine['region_type'] == type_r]
        #print(len(x))
        if permute == True:
            random_per = []
            for j in range(len(x)):
                random_per.append(random.uniform(-0.3,0.3))
            plt.scatter(x[:, 0] + np.array(random_per) , x[:, 1], c = color_use[i], marker='o', s=10, label=type_r)
        else:
            plt.scatter(x[:, 0], x[:, 1], c = color_use[i], marker='o', s=10, label=type_r)
    plt.legend(fontsize = 12, frameon = False)
    plt.xlabel('length',  fontsize = 12)
    plt.ylabel('Average Score',  fontsize = 12)
    plt.xticks(fontsize = 12)
    plt.yticks(fontsize = 12)
    #plt.ylim([0,1])
    ax=plt.gca()
    ax.spines['bottom'].set_linewidth(1.6)
    ax.spines['left'].set_linewidth(1.6)
    ax.spines['right'].set_linewidth(1.6)
    ax.spines['top'].set_linewidth(1.6)
    ax.tick_params(axis = 'y', length=7, width = 1.6)
    ax.tick_params(axis = 'x', length=3, width = 1.6)   
    if save_name != '':
        plt.savefig(save_name, format = 'svg', transparent = True) 
    #plt.show()
    #fig = plt.gcf()
    #plt.close(fig)
    ''' 
    return df_boundary_region_combine 
  
    
def AdjustBoundaryTypesByCutoff(df_boundary_region_with_type, color_use, save_name = '', permute=True):    
    """
    Adjust boundary region type by setting cut-off of boundary region length and average score 
    based on the K-means results

    Parameters
    ----------
    df_boundary_region_with_type : pandas.DataFrame
        DataFrame contains the boundary region with types.
    color_use : list
        Color used for drawing scatter plot of boundary regions.
    save_name : str, optional
        save path and name of figure. The default is ''.
    permute : bool, optional
        Whether permute scatters to avoid overlap. The default is True.

    Returns
    -------
    df_boundary_region_with_type : pandas.DataFrame
        DataFrame contains the adjusted boundary region types.

    """
    df_wide_part = df_boundary_region_with_type[df_boundary_region_with_type['region_type'] == 'Wide']
    cut_length = np.percentile(df_wide_part['length'], 10)    
    df_sharp_strong_part = df_boundary_region_with_type[df_boundary_region_with_type['region_type'] == 'Narrow-strong']    
    cut_score = np.percentile(df_sharp_strong_part['ave_score'], 10)
    cut_score = np.ceil(cut_score)
    print('Length cut-off:' + str(cut_length))
    print('Score cut-off:' + str(cut_score))
    region_type_adjust = []
    for i in range(len(df_boundary_region_with_type)):
        length = df_boundary_region_with_type['length'][i]
        score = df_boundary_region_with_type['ave_score'][i]
        if length >= cut_length:
            region_type_adjust.append('Wide')
        elif score >= cut_score:
            region_type_adjust.append('Narrow-strong')
        else:
            region_type_adjust.append('Narrow-weak')
    df_boundary_region_with_type['region_type_adjust'] = region_type_adjust
    #print('Adjust number:' + str(np.sum(df_boundary_region_with_type['region_type_adjust'] != df_boundary_region_with_type['region_type'])))
    
    plt.figure(figsize=(5, 4.2))
    X = np.array(df_boundary_region_with_type[['length', 'ave_score']])
    type_list_r = ['Narrow-strong', 'Narrow-weak', 'Wide']
    for i in range(len(type_list_r)) :
        type_r = type_list_r[i]
        x = X[df_boundary_region_with_type['region_type_adjust'] == type_r]
        print('Boundary region types: ' + type_r + ', Number: ' + str((len(x))))
        if permute == True:
            random_per = []
            for j in range(len(x)):
                random_per.append(random.uniform(-0.3,0.3))
            plt.scatter(x[:, 0] + np.array(random_per) , x[:, 1], c = color_use[i], marker='o', s=10, label=type_r)
        else:
            plt.scatter(x[:, 0], x[:, 1], c = color_use[i], marker='o', s=10, label=type_r)
    plt.legend(fontsize = 12, frameon = False)
    plt.xlabel('length (#bins)',  fontsize = 12)
    plt.ylabel('Average Score',  fontsize = 12)
    plt.xticks(fontsize = 12)
    plt.yticks(fontsize = 12)
    #plt.ylim([0,1])
    ax=plt.gca()
    ax.spines['bottom'].set_linewidth(1.6)
    ax.spines['left'].set_linewidth(1.6)
    ax.spines['right'].set_linewidth(1.6)
    ax.spines['top'].set_linewidth(1.6)
    ax.tick_params(axis = 'y', length=7, width = 1.6)
    ax.tick_params(axis = 'x', length=3, width = 1.6)   
    if save_name != '':
        plt.savefig(save_name, format = 'svg', transparent = True) 
    plt.show()
    fig = plt.gcf() #获取当前figure
    #plt.close(fig)  
    return df_boundary_region_with_type
            

def IdentifyBoundaryRegions(df_boundary_region_combine, K, color_use, save_name = '', permute=True):
    """
    Identify three types of boundary regions with Kmeans 
    based on the length and average scores of boundary regions 

    Parameters
    ----------
    df_boundary_region_combine : pandas.DataFrame
        DataFrame contains the boundary region defined from the boundary score profile.
    K : int
        Number of clusters (Types of boundary regions).
    color_use : list
        Color used for drawing scatter plot of boundary regions.
    save_name : str, optional
        save path and name of figure. The default is ''.
    permute : bool, optional
        Whether permute scatters to avoid overlap. The default is True.
   
    Returns
    -------
    df_boundary_region_with_types : pandas.DataFrame
        DataFrame contains the boundary region with types.
    """
    df_boundary_region_types = GetBoundaryRegionsWithKmeans(df_boundary_region_combine, K, color_use, save_name = '', permute=True)
    df_boundary_region_with_types = AdjustBoundaryTypesByCutoff(df_boundary_region_types, color_use, save_name = '', permute=True)
    return df_boundary_region_with_types

























# -*- coding: utf-8 -*-
"""
Created on Fri May 20 22:37:13 2022

@author: dcdang
"""

import sys
import copy
import numpy as np

from . import Preprocess as pre
from . import TadSeparationLandscape as TSL
from . import IdentifyBoundaryRegions as IBR
from . import GetConsTADs as GCT

#src_path = 'E:/Users/dcdang/project/monkey project/TAD_intergare/ConsTADs_script/scripts'
#if src_path not in sys.path:
    #sys.path.append(src_path)

def ConsTADs(TAD_caller_result_add, target_chr, resolution, chr_size, method_list,
             mat_file, mat_type, window_list, mat_norm_check = False, p_cut = 0.05, high_score_cut = 5, combine_dist = 2,
             K = 3, color_bd = ['#D65F4D', '#459457', '#4392C3'], 
             weight = 0.5):
    """
    

    Parameters
    ----------
    TAD_caller_result_add : str
        Path of fold containing results of multiple TAD callers.
    target_chr : str
        Symbol of target chromosome, eg: chr2.
    resolution : int
        Resolution of Hi-C contact map.
    chr_size : int
        Length of target chromosome in base pair.
    method_list : list
        List of TAD caller names.
    mat_file : str
        Path of file for Hi-C contact map of target chr.
    mat_type : str
        Type of Hi-C contact map file, dense or sparse.
    window_list : list
        List of window size for calculating multi-scale contrast p-value. These sizes are lengths in base pair. 
    mat_norm_check : bool, optional
        Dose the Hi-C contact map are distance-dependent normalized. The default is False. 
    p_cut : float, optional
        Cut off of contrast p-value. The default is 0.05.
    high_score_cut : int, optional
        Cut off of confident boundary score. The default is 5.
    combine_dist : int, optional
        The cut off of distance for boundary regions' combination. The default is 2.
    K : int, optional
        Number of boundary region types. The default is 3.
    color_bd : list, optional
        List of color for different boundary region types. The default is ['#D65F4D', '#459457', '#4392C3'].
    weight : float, optional
        weight to calculate best bin pair. 
        Balance affect of boundary score and domain value. The default is 0.5 for boundary score.

    Returns
    -------
    TAD_caller_result_all : dict
        Dictionary containing the results of multiple TAD callers for the Hi-C contact map of target chromosme.
    mat_dense : numpy.array
        Hi-C contact map in dense format.
    mat_norm : numpy.array
        Distance-dependent normalized Hi-C contact map in dense format.
    result_record : dict
        Result of each oreration when building the TAD separation landscape.
    w_best : int
        Best window size for contrast p-value.
    df_bd_insul_pvalue : pandas.DataFrame
        Multi-scale contrast p-value for each bin along the chromosome.
    df_pvalue_score_cor : pandas.DataFrame
        Pearson correlation of multi-scale contrast p-value and boundary score profile.
    df_boundary_region_with_types : pandas.DataFrame
        DataFrame of Boundary regions and their types.
    df_tad_cons : pandas.DataFrame
        DataFrame of ConsTADs.
    df_boundary_cons : pandas.DataFrame
        DataFrame of ConsTADs boundaries.

    """
    bin_num = int(np.ceil(chr_size / resolution))
    print( '\033[1mStep 1: Preprocess of TAD caller results...\033[0m' )
    TAD_caller_result_all = pre.PreprocessTADs(TAD_caller_result_add, method_list, resolution, target_chr, chr_size)
    print('Done!')
    
    print('\033[1mStep 2: Boundary voting...\033[0m')
    bd_score_primary = TSL.BoundaryVoting(TAD_caller_result_all, method_list, bin_num, chr_size, target_chr, resolution, expand_bin = 0)
    print('Done!')
    
    print('\033[1mStep 3: Build TAD separation landscape...\033[0m')
    mat_dense, mat_norm, result_record, w_best, df_bd_insul_pvalue, df_pvalue_score_cor = TSL.BuildTadSeperationLandscape(target_chr, resolution, mat_file, bin_num, mat_type, mat_norm_check, bd_score_primary, window_list, 
                                   p_cut, high_score_cut, combine_dist, norm_type = 'z-score', cut_dist = 12000000)
    print('Done!')
   
    print('\033[1mStep 4: Identify three types of boundary regions...\033[0m')
    df_boundary_region_combine = copy.deepcopy(result_record['Combine']['bd_region'])
    df_boundary_region_with_types = IBR.IdentifyBoundaryRegions(df_boundary_region_combine, K, color_bd, save_name = '', permute=True)
    print('Done!')
    
    print('\033[1mStep 5: Get ConsTADs based on TAD separation landscape...\033[0m')
    df_tad_cons, df_boundary_cons = GCT.BuildConsTADs(target_chr, resolution, mat_dense, df_boundary_region_with_types, weight)
    print('Done!')
    
    return TAD_caller_result_all, mat_dense, mat_norm, result_record, w_best, df_bd_insul_pvalue, df_pvalue_score_cor, df_boundary_region_with_types, df_tad_cons, df_boundary_cons



















# -*- coding: utf-8 -*-
"""
Created on Sat Mar 19 15:33:14 2022

@author: dcdang
"""

import os
import pandas as pd
import numpy as np
import pickle
import scipy
import copy



def SaveData(file, objects):
    save_file = open(file, 'wb')
    pickle.dump(objects, save_file, 2)


def ReadSaveData(file):
    read_file = open(file, 'rb')
    objects = pickle.load(read_file)
    read_file.close()
    return objects


def SparseMatrixToDense(df_mat_sparse, bin_num):
    df_mat_sparse.columns = ['bin1', 'bin2', 'value']
    row = np.array(df_mat_sparse['bin1'])
    col = np.array(df_mat_sparse['bin2'])
    val = np.array(df_mat_sparse['value'])
    mat_hic_sparse = scipy.sparse.csr_matrix((val, (row,col)), shape = (bin_num, bin_num))
    mat_dense_up = mat_hic_sparse.toarray()
    mat_dense_low = mat_dense_up.T
    mat_dense_diag = np.diag(np.diag(mat_dense_up))
    mat_dense = mat_dense_up + mat_dense_low - mat_dense_diag
    return mat_dense
    

def LoadHicMat(mat_file, bin_num, mat_type = 'dense'):
    """
    Load Hi-C data dense matrix. The default matrix file is in .csv format
    with \t separation.

    Parameters
    ----------
    mat_file : str
        File path of Hi-C data matrix.
    mat_type : str, optional
        Type of matrix file, dense or sparse. The default is 'dense'.

    Returns
    -------
    mat_hic : numpy.array
        Hi-C data dense matrix
    """
    if os.path.exists(mat_file) == False:
        print('Hi-C matrix do not exit!')
    if mat_type == 'dense':
        df_mat_dense = pd.read_csv(mat_file, sep = '\t', header = None)
        mat_hic = np.array(df_mat_dense.values)
    if mat_type == 'sparse':
        df_mat_sparse = pd.read_csv(mat_file, sep = '\t', header = None)
        mat_hic = SparseMatrixToDense(df_mat_sparse, bin_num) 
    return mat_hic


def DistanceNormalizedMatrix(mat_target, resolution, norm_type = 'z-score', cut_dist = 5000000):
    """
    Normalize Hi-C data matrix by each diagonal.

    Parameters
    ----------
    mat_target : numpy.array
        Input Hi-C data matrix.
    resolution : int
        Hi-C data resolution.
    norm_type : str, optional
        Type of normalization, including z-score, min-max, obs_exp. 
        The default is 'z-score'.
    cut_dist : int, optional
        Range of Hi-C data to normalize, represent bins within the distance.
        The default is 5000000.

    Returns
    -------
    mat_zscore : numpy.array
        Normalized Hi-C data matrix.

    """
    mat_normalize = mat_target
    mat_zscore = np.zeros([len(mat_normalize), len(mat_normalize)])
    cut = int(cut_dist / resolution)
    cut = np.min([cut, len(mat_zscore)])
    for i in range(cut):
        diag_value = np.diag(mat_normalize, i)
        if norm_type == 'z-score':
            if np.std(diag_value) == 0:
                diag_z_score = diag_value - np.mean(diag_value)
            else:
                diag_z_score = (diag_value - np.mean(diag_value)) / np.std(diag_value)
        elif norm_type == 'min-max':
            if np.max(diag_value) == np.min(diag_value):
                diag_z_score = diag_value - np.min(diag_value)
            else:
                diag_z_score = (diag_value - np.min(diag_value)) / (np.max(diag_value) - np.min(diag_value))
        elif norm_type == 'obs_exp':
            diag_value_copy = copy.deepcopy(diag_value)
            diag_value_copy[diag_value == 0] = np.mean(diag_value)
            diag_z_score = np.log2((diag_value_copy / np.mean(diag_value)))
        mat_zscore += np.diag(diag_z_score, k = i)
    mat_zscore = mat_zscore + mat_zscore.T - np.diag(np.diag(mat_zscore, 0), k = 0)   
    return mat_zscore























# -*- coding: utf-8 -*-
"""
Created on Fri Mar 18 15:57:09 2022

@author: dcdang
"""

import os
import pandas as pd
import numpy as np
import copy


def __get_TAD_boundary_result_for_basic(method, df_result, target_chr, resolution, chr_size):
    df_result.columns = ['st', 'ed']
    df_TAD = pd.DataFrame(columns = ['chr', 'start', 'end', 'name', 'boundary_st', 'boundary_ed'])
    df_boundary = pd.DataFrame(columns = ['chr', 'start', 'end', 'TAD_name'])
    TAD_st_list = (df_result['st'] - 1) * resolution 
    TAD_ed_list = df_result['ed'] * resolution
    last = False
    if TAD_ed_list[len(TAD_ed_list)-1] > chr_size:
        hold = copy.deepcopy(TAD_ed_list[len(TAD_ed_list)-1])
        last = True
        TAD_ed_list[len(TAD_ed_list)-1] = chr_size
    df_TAD['start'] = TAD_st_list    
    df_TAD['end'] = TAD_ed_list 
    df_TAD['chr'] = [target_chr for i in range(len(df_TAD))]
    df_TAD = df_TAD.sort_values(by = ['chr', 'start', 'end'])
    df_TAD = df_TAD.reset_index(drop = True)
    ### filter wrong TAD
    df_TAD = df_TAD[(df_TAD['end'] - df_TAD['start']) > 2 * resolution]
    df_TAD = df_TAD.reset_index(drop = True)
    
    df_TAD['name'] = [method + '_' + target_chr + '_TAD_' + str(i) for i in range(len(df_TAD))]
    df_TAD['boundary_st'] = [target_chr + ':' + str(df_TAD['start'][i]) + '-' +  
          str(df_TAD['start'][i] + resolution) for i in range(len(df_TAD))]
    df_TAD['boundary_ed'] = [target_chr + ':' + str(df_TAD['end'][i] - resolution) + '-' +  
          str(df_TAD['end'][i]) for i in range(len(df_TAD))]
    if last == True:
        bd_l = list(df_TAD['boundary_ed'])
        bd_l[-1] = target_chr + ':' + str(hold - resolution) + '-' + str(chr_size)
        df_TAD['boundary_ed'] = bd_l
        #df_TAD['boundary_ed'].iloc[-1] = target_chr + ':' + str(hold - resolution) + '-' + str(chr_size)
    df_boundary['start'] = list(df_TAD['start']) +  list(df_TAD['end'] - resolution)
    if last == True:
        st_l = list(df_boundary['start'])
        st_l[-1] = hold - resolution 
        df_boundary['start'] = st_l
        #df_boundary['start'].iloc[-1] = hold - resolution    
    df_boundary['end'] = list(df_TAD['start'] + resolution) +  list(df_TAD['end'])
    df_boundary['chr'] = [target_chr for i in range(len(df_boundary))]
    df_boundary['TAD_name'] = list(df_TAD['name']) + list(df_TAD['name'])
    df_boundary = df_boundary.sort_values(by = ['chr','start', 'end'])
    df_boundary = df_boundary.reset_index(drop = True)
    return df_TAD, df_boundary


def __get_low_level_domain_3DNet(df_result):
    judge_l = []
    for i in range(len(df_result)):
        st = df_result['st'][i]
        ed = df_result['ed'][i]
        if i == 0:
            judge_l.append('Yes')
            hold_ind = i
            hold_st = st
            hold_ed = ed
            continue
        if hold_st <= st < hold_ed and hold_ed < ed:
            judge_l.append('No')
        elif hold_st <= st < hold_ed and hold_st < ed <= hold_ed:
            judge_l[hold_ind] = 'No'
            judge_l.append('Yes')
            hold_ind = i
            hold_st = st
            hold_ed = ed
        elif st >= hold_ed:
            judge_l.append('Yes')
            hold_ind = i
            hold_st = st
            hold_ed = ed            
    df_result_part = copy.deepcopy(df_result)
    df_result_part['judge'] = judge_l
    df_result_part = df_result_part[df_result_part['judge'] == 'Yes']
    df_result_part = df_result_part.reset_index(drop = True)
    return df_result_part
        

def __get_TAD_boundary_result_for_3DNetMod(method, df_result, target_chr, resolution, chr_size, part = True):
    df_result = df_result.sort_values(by = [1,2])
    df_result = df_result.reset_index(drop = True)
    df_result[1] = np.array(df_result[1] / resolution).astype('int32')
    df_result[2] = np.array(df_result[2] / resolution).astype('int32')    
    df_result.columns = ['chr', 'st', 'ed']
    if part == True:
        df_result = __get_low_level_domain_3DNet(df_result)    
    df_result_new = pd.DataFrame(columns = ['st', 'ed'])
    df_result_new['st'] = df_result['st'] + 1
    df_result_new['ed'] = df_result['ed']
    df_TAD, df_boundary = __get_TAD_boundary_result_for_basic(method, df_result_new, target_chr, resolution, chr_size)
    return df_TAD, df_boundary
  
      
def __get_TAD_boundary_result_for_CaTCH(method, df_result, target_chr, resolution, chr_size):
    df_result.columns = ['index', 'chr', 'RI', 'start', 'end', 'IS']
    df_result_new = pd.DataFrame(columns = ['st', 'ed'])
    df_result_new['st'] = copy.deepcopy(df_result['start'])
    df_result_new['ed'] = copy.deepcopy(df_result['end'])
    df_TAD, df_boundary = __get_TAD_boundary_result_for_basic(method, df_result_new, target_chr, resolution, chr_size)
    return df_TAD, df_boundary


def __get_TAD_boundary_result_for_CHDF(method, df_result, target_chr, resolution, chr_size):
    df_result.columns = ['start', 'end']
    df_result_new = pd.DataFrame(columns = ['st', 'ed'])
    df_result_new['st'] = copy.deepcopy(df_result['start'])
    df_result_new['ed'] = copy.deepcopy(df_result['end'])
    df_TAD, df_boundary = __get_TAD_boundary_result_for_basic(method, df_result_new, target_chr, resolution, chr_size)
    return df_TAD, df_boundary


def __get_TAD_boundary_result_for_ClusterTAD(method, df_result, target_chr, resolution, chr_size):
    df_result.columns = ['start', 'st_cor', 'end', 'ed_cor']
    df_result_new = pd.DataFrame(columns = ['st', 'ed'])
    df_result_new['st'] = df_result['start'] + 1
    df_result_new['ed'] = df_result['end']
    df_TAD, df_boundary = __get_TAD_boundary_result_for_basic(method, df_result_new, target_chr, resolution, chr_size)
    return df_TAD, df_boundary


def __get_TAD_boundary_result_for_deDoc(method, df_result, target_chr, resolution, chr_size):
    df_result_new = pd.DataFrame(columns = ['st', 'ed'])
    st_list = []
    ed_list = []
    for i in range(len(df_result)):
       st = int(df_result.iloc[i][0].split(' ')[0])
       ed = int(df_result.iloc[i][0].split(' ')[-1])
       st_list.append(st)
       ed_list.append(ed)
    df_result_new['st'] = st_list   
    df_result_new['ed'] = ed_list     
    df_TAD, df_boundary = __get_TAD_boundary_result_for_basic(method, df_result_new, target_chr, resolution, chr_size)
    return df_TAD, df_boundary


def __get_TAD_boundary_result_for_DI(method, df_result, target_chr, resolution, chr_size):
    df_result.columns = ['chr', 'start', 'end']
    df_result_new = pd.DataFrame(columns = ['st', 'ed'])
    st_ser= df_result['start'] / resolution
    ed_ser= df_result['end'] / resolution
    st_ser = st_ser.astype(np.int32)
    ed_ser = ed_ser.astype(np.int32)
    # DI result is 0-based, but end need not add 1, becaues the input is loc,not index.
    df_result_new['st'] = st_ser + 1
    df_result_new['ed'] = ed_ser
    df_TAD, df_boundary = __get_TAD_boundary_result_for_basic(method, df_result_new, target_chr, resolution, chr_size)
    return df_TAD, df_boundary


def __get_TAD_boundary_result_for_GMAP(method, df_result, target_chr, resolution, chr_size):
    df_result.columns = ['start', 'end']
    df_result = df_result.sort_values(by = ['start', 'end'])
    df_result = df_result.reset_index(drop = True)
    df_result_new = pd.DataFrame(columns = ['st', 'ed'])
    df_result_new['st'] = df_result['start'] - int(resolution / 2)
    df_result_new['ed'] = df_result['end'] + int(resolution / 2)    
    st_ser= df_result_new['st'] / resolution
    ed_ser= df_result_new['ed'] / resolution
    st_ser = st_ser.astype(np.int32)
    ed_ser = ed_ser.astype(np.int32)    
    df_result_new['st'] = st_ser + 1
    df_result_new['ed'] = ed_ser - 1
    df_TAD, df_boundary = __get_TAD_boundary_result_for_basic(method, df_result_new, target_chr, resolution, chr_size)
    return df_TAD, df_boundary


def __get_TAD_boundary_result_for_HiCDB(method, df_result, target_chr, resolution, chr_size):
    df_result.columns = ['chr', 'st_bd', 'ed_bd', 'pvalue', 'CI']
    df_result = df_result.sort_values(by = ['st_bd', 'ed_bd'])
    df_result = df_result.reset_index(drop = True)    
    st_l = []
    ed_l = []
    for i in range(len(df_result)-1):
        st = int(df_result['ed_bd'][i] / resolution)
        ed = int(df_result['st_bd'][i+1] / resolution)
        st_l.append(st)
        ed_l.append(ed)
    df_result_new = pd.DataFrame(columns = ['st', 'ed'])
    df_result_new['st'] = st_l
    df_result_new['ed'] = ed_l
    df_TAD, df_boundary = __get_TAD_boundary_result_for_basic(method, df_result_new, target_chr, resolution, chr_size)
    return df_TAD, df_boundary


def __get_TAD_boundary_result_for_Hiseg(method, df_result, target_chr, resolution, chr_size):
    df_result.columns = ['segment']
    df_result = df_result.reset_index(drop = True)
    df_result_new = pd.DataFrame(columns = ['st', 'ed'])
    st_list = []
    ed_list = []
    # Hiseg is R method, 1-based
    for i in range(len(df_result) - 1):
        if i != len(df_result) - 2:
            st_list.append(df_result['segment'][i])
            ed_list.append(df_result['segment'][i+1]-1)
        else:
            st_list.append(df_result['segment'][i])
            ed_list.append(df_result['segment'][i+1])           
    df_result_new['st'] = st_list
    df_result_new['ed'] = ed_list
    df_TAD, df_boundary = __get_TAD_boundary_result_for_basic(method, df_result_new, target_chr, resolution, chr_size)
    return df_TAD, df_boundary
 
    
def __get_TAD_boundary_result_for_HiTAD(method, df_result, target_chr, resolution, chr_size, part = True):   
    df_result.columns = ['chr', 'st', 'ed', 'TAD_level']
    df_result = df_result.sort_values(by = [ 'st', 'ed'])
    df_result = df_result.reset_index(drop = True)
    if part == True:
        df_result_part = __get_low_level_domain_3DNet(df_result)    
        df_result = df_result_part
    df_result_new = pd.DataFrame(columns = ['st', 'ed'])
    st_ser= df_result['st'] / resolution
    ed_ser= df_result['ed'] / resolution
    st_ser = st_ser.astype(np.int32)
    ed_ser = ed_ser.astype(np.int32)
    # DI result is 0-based
    df_result_new['st'] = st_ser + 1
    df_result_new['ed'] = ed_ser
    df_TAD, df_boundary = __get_TAD_boundary_result_for_basic(method, df_result_new, target_chr, resolution, chr_size)
    df_TAD['TAD_level'] = df_result['TAD_level']
    return df_TAD, df_boundary


def __get_TAD_boundary_result_for_ICFinder(method, df_result, target_chr, resolution, chr_size):
    df_TAD, df_boundary = __get_TAD_boundary_result_for_basic(method, df_result, target_chr, resolution, chr_size)
    return df_TAD, df_boundary

    
def __get_TAD_boundary_result_for_IS_old(method, df_result, target_chr, resolution, chr_size):
    boundary_list = df_result['header.1']
    segment_list = []
    for i in range(len(boundary_list)):
        end = int(boundary_list[i].split('-')[-1]) - 1
        segment_list.append(int(end / resolution))
        df_result_new = pd.DataFrame(segment_list)
    df_TAD, df_boundary = __get_TAD_boundary_result_for_Hiseg(method, df_result_new, target_chr, resolution, chr_size)
    return df_TAD, df_boundary


def __get_TAD_boundary_result_for_IS(method, df_result, target_chr, resolution, chr_size):
    df_result.columns = ['chr', 'range_st', 'range_ed', 'header', 'IS_strength']
    boundary_list = df_result['header']
    segment_list = []
    for i in range(len(boundary_list)):
        end = int(boundary_list[i].split('-')[-1]) - 1
        segment_list.append(int(end / resolution))
        df_result_new = pd.DataFrame(segment_list)
    df_TAD, df_boundary = __get_TAD_boundary_result_for_Hiseg(method, df_result_new, target_chr, resolution, chr_size)
    return df_TAD, df_boundary


def __get_TAD_boundary_result_for_MSTD(method, df_result, target_chr, resolution, chr_size):
    df_result.columns = ['start', 'end', 'center']
    df_result_new = pd.DataFrame(columns = ['st', 'ed'])
    df_result_new['st'] = df_result['start'] + 1
    df_result_new['ed'] = df_result['end'] + 1 - 1
    df_TAD, df_boundary = __get_TAD_boundary_result_for_basic(method, df_result_new, target_chr, resolution, chr_size)
    return df_TAD, df_boundary

    
def __get_TAD_boundary_result_for_OnTAD(method, df_result, target_chr, resolution, chr_size, part = True):   
    df_result.columns = ['st', 'ed', 'TAD_level', 'score1', 'score2']
    df_result = df_result.sort_values(by = [ 'st', 'ed'])
    df_result = df_result.reset_index(drop = True)
    if df_result['st'][0] == 1 and df_result['ed'][0] == np.ceil(chr_size / resolution):
       df_result = df_result.iloc[1:]
       df_result = df_result.reset_index(drop = True)
    if part == True:
        df_result_part = __get_low_level_domain_3DNet(df_result)    
        df_result = df_result_part
    df_result_new = pd.DataFrame(columns = ['st', 'ed'])
    df_result_new['st'] = df_result['st']
    df_result_new['ed'] = df_result['ed']-1
    df_TAD, df_boundary = __get_TAD_boundary_result_for_basic(method, df_result_new, target_chr, resolution, chr_size)
    df_TAD['TAD_level'] = df_result['TAD_level']
    return df_TAD, df_boundary


def __get_TAD_boundary_result_for_Spectral(method, df_result, target_chr, resolution, chr_size):
    df_result.columns = ['info']
    df_result_new = pd.DataFrame(columns = ['chr', 'start', 'end'])
    chr_list = [target_chr for i in range(len(df_result))]
    st_list = []
    ed_list = []
    for i in range(len(df_result)):
        target = df_result['info'][i]
        st = int(target.split('-')[0])
        ed = int(target.split('-')[-1])
        st_list.append(st)
        ed_list.append(ed)
    df_result_new['chr'] = chr_list
    df_result_new['start'] = st_list
    df_result_new['end'] = ed_list
    df_TAD, df_boundary = __get_TAD_boundary_result_for_DI(method, df_result_new, target_chr, resolution, chr_size)
    return df_TAD, df_boundary


def __get_TAD_boundary_result_for_TopDom(method, df_result, target_chr, resolution, chr_size):
    df_result_part = df_result[df_result['tag'] == 'domain']
    df_result_part = df_result_part.reset_index(drop = True)
    df_result_new = pd.DataFrame(columns = ['st', 'ed'])
    df_result_new['st'] = df_result_part['from.id']
    df_result_new['ed'] = df_result_part['to.id']    
    df_TAD, df_boundary = __get_TAD_boundary_result_for_basic(method, df_result_new, target_chr, resolution, chr_size)
    return df_TAD, df_boundary


def PreprocessTADs(TAD_caller_result_add, method_list, resolution, target_chr, chr_size):
    """
    Preprocess of TAD caller results to get the uniform format of 
    TAD domain and TAD boundary.

    Parameters
    ----------
    TAD_caller_result_add : str
        Path to the folder containing TAD Caller Results
    method_list : list
        List of TAD caller
    resolution : int
        Resolution of Hi-C data 
    target_chr : str
        Symbol of used chromosome 
    chr_size : int
        Length of used chromosome (bp).

    Returns
    -------
    TAD_caller_result_all : dict
        Dictionary contains the uniform format of TAD domain and TAD boundary
    """
    if os.path.exists(TAD_caller_result_add) == False:
        print('TAD caller results do not exit!')
    TAD_caller_result_all = {}
    for method in method_list:
        TAD_caller_result_all[method] = {}
        file_name = method + '_Result.txt'
        file = TAD_caller_result_add + '/' + file_name
        if os.path.exists(file) == False:
            print('Can not find TADs of ' + method)
            continue
        if method in ['GMAP', 'HiCseg']:
            df_result = pd.read_csv(file, sep = ' ', header = 0)
        elif method in ['TADtree', 'TopDom', 'MSTD', 'ClusterTAD']:
            df_result = pd.read_csv(file, sep = '\t', header = 0)
        elif method in ['IS']:
            df_result = pd.read_csv(file, sep = '\t', header = None, skiprows=1)
        elif method in ['CaTCH']:
            df_result = pd.read_csv(file, sep = ' ', header = None)
        else:         
            df_result = pd.read_csv(file, sep = '\t', header = None)
        if method == '3DNetMod':
            df_TAD, df_boundary = __get_TAD_boundary_result_for_3DNetMod(method, df_result, target_chr, resolution, chr_size, part = True)
        if method == 'CaTCH':
            df_TAD, df_boundary = __get_TAD_boundary_result_for_CaTCH(method, df_result, target_chr, resolution, chr_size)
        if method == 'CHDF':
            df_TAD, df_boundary = __get_TAD_boundary_result_for_CHDF(method, df_result, target_chr, resolution, chr_size)
        if method == 'ClusterTAD':
            df_TAD, df_boundary = __get_TAD_boundary_result_for_ClusterTAD(method, df_result, target_chr, resolution, chr_size)
        if method == 'deDoc':
            df_TAD, df_boundary = __get_TAD_boundary_result_for_deDoc(method, df_result, target_chr, resolution, chr_size)
        if method == 'DI':
            df_TAD, df_boundary = __get_TAD_boundary_result_for_DI(method, df_result, target_chr, resolution, chr_size)
        if method =='GMAP':
            df_TAD, df_boundary = __get_TAD_boundary_result_for_GMAP(method, df_result, target_chr, resolution, chr_size)
        if method =='HiCDB':
            df_TAD, df_boundary = __get_TAD_boundary_result_for_HiCDB(method, df_result, target_chr, resolution, chr_size)
        if method =='HiCseg':
            df_TAD, df_boundary = __get_TAD_boundary_result_for_Hiseg(method, df_result, target_chr, resolution, chr_size)
        if method == 'HiTAD':
            df_TAD, df_boundary = __get_TAD_boundary_result_for_HiTAD(method, df_result, target_chr, resolution, chr_size, part = True)
        if method =='ICFinder':
            df_TAD, df_boundary = __get_TAD_boundary_result_for_ICFinder(method, df_result, target_chr, resolution, chr_size)
        if method =='IS':
            df_TAD, df_boundary = __get_TAD_boundary_result_for_IS(method, df_result, target_chr, resolution, chr_size)
        if method =='MSTD':
            df_TAD, df_boundary = __get_TAD_boundary_result_for_MSTD(method, df_result, target_chr, resolution, chr_size)
        if method == 'Spectral':
             df_TAD, df_boundary = __get_TAD_boundary_result_for_Spectral(method, df_result, target_chr, resolution, chr_size)
        if method =='OnTAD':
            df_TAD, df_boundary = __get_TAD_boundary_result_for_OnTAD(method, df_result, target_chr, resolution, chr_size, part = True)
        if method =='TopDom':
            df_TAD, df_boundary = __get_TAD_boundary_result_for_TopDom(method, df_result, target_chr, resolution, chr_size)        
        TAD_caller_result_all[method]['TAD_domain'] = df_TAD
        TAD_caller_result_all[method]['TAD_boundary'] = df_boundary
    return TAD_caller_result_all

     
























from setuptools import Command, find_packages, setup

__lib_name__ = "ConsTADs"
__lib_version__ = "1.0.0"
__description__ = "Defining the separation landscape of topological domains for decoding consensus domain organization of 3D genome"
__url__ = "https://github.com/zhanglabtools/ConsTADs"
__author__ = "Dachang Dang"
__author_email__ = "dangdachang@163.com"
__license__ = "MIT"
__keywords__ = ["3D Genome", "topological domains"]
__requires__ = ["requests",]

with open("README.rst", "r", encoding="utf-8") as f:
    __long_description__ = f.read()

setup(
    name = __lib_name__,
    version = __lib_version__,
    description = __description__,
    url = __url__,
    author = __author__,
    author_email = __author_email__,
    license = __license__,
    packages = ['ConsTADs'],
    install_requires = __requires__,
    zip_safe = False,
    include_package_data = True,
    long_description = __long_description__
)


# -*- coding: utf-8 -*-
"""
Created on Sat May 21 12:45:18 2022

@author: dcdang
"""


import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.axes_grid1.inset_locator import inset_axes
from matplotlib.colors import ListedColormap
import matplotlib
import copy
import matplotlib.colors as mcolors


def DrawBdScoreMultiScalePvalue(mat_dense, Chr, st, ed, bd_cell_score_original, bd_cell_score_refined, df_pvalue_multi, window_l, resolution, save_name, p_cut, target_site = [], fgsize = (12, 8), bin_size = 10):
    plt.figure(figsize=(fgsize[0], fgsize[1]))
    x_axis_range = range(len(bd_cell_score_original['bd_score'][st:ed]))
    
    start_ =  st * resolution 
    end_ = ed * resolution

    cord_list = []
    pos_list = []
    pos_start = start_ 
    x_ticks_l = []
    y_ticks_l = []
    for i in range(ed - st):
        if i % bin_size == 0:
            cord_list.append(i)
            pos = pos_start + i*resolution
            pos_list.append(pos)
            if i + bin_size < ed - st:
                pos_label = str(pos / 1000000)
            else:
                #pos_label = str(pos / 1000000) + '(Mb)'
                pos_label = str(pos / 1000000) 
            x_ticks_l.append(pos_label)
            y_ticks_l.append(str(pos / 1000000))
    region_name = Chr + ':' + str(start_ / 1000000) + '-' + str(end_ / 1000000) + ' Mb'
         
    ax1 = plt.subplot2grid((12, 7), (0, 0), rowspan=6,colspan=6)
    start = int(start_ / resolution)
    end = int(end_ / resolution)
    dense_matrix_part = mat_dense[start:end, start:end]
    img = ax1.imshow(dense_matrix_part, cmap='coolwarm', vmin = np.percentile(dense_matrix_part, 10), 
                     vmax = np.percentile(dense_matrix_part, 90))
    
    ax1.set_xticks([])
    ax1.spines['bottom'].set_linewidth(0)
    ax1.spines['left'].set_linewidth(1.6)
    ax1.spines['right'].set_linewidth(0)
    ax1.spines['top'].set_linewidth(0)
    ax1.tick_params(axis = 'y', length=5, width = 1.6)
    ax1.tick_params(axis = 'x', length=5, width = 1.6)
    #plt.xticks(cord_list, x_ticks_l, fontsize = 0,  rotation = 90)
    plt.xticks(cord_list, ['' for k in range(len(cord_list))], fontsize = 0, rotation = 90)
    plt.yticks(cord_list, y_ticks_l, fontsize = 10)
    ax1.set_title('TAD landscape of region:' + region_name, fontsize=12, pad = 15.0)

    cax = plt.subplot2grid((12, 7), (0, 6), rowspan=6,colspan=1)
    cbaxes = inset_axes(cax, width="30%", height="94%", loc=3) 
    plt.colorbar(img, cax = cbaxes, orientation='vertical')
    cax.spines['bottom'].set_linewidth(0)
    cax.spines['left'].set_linewidth(0)
    cax.spines['right'].set_linewidth(0)
    cax.spines['top'].set_linewidth(0)
    cax.tick_params(axis = 'y', length=0, width = 0)
    cax.tick_params(axis = 'x', length=0, width = 0)
    cax.set_xticks([])
    cax.set_yticks([])
    
    for i in range(1, len(window_l)+1):
        wd = window_l[-i]
        ax1_5 = plt.subplot2grid((12, 7), (6 + i - 1, 0), rowspan=1,colspan=6, sharex=ax1)
        ax1_5.plot(list(df_pvalue_multi[wd][st:ed]), color='black')
        ax1_5.bar(x_axis_range, list(df_pvalue_multi[wd][st:ed]), label=wd, color='#B4E0F9')
        if p_cut != 0:
            ax1_5.hlines(p_cut, x_axis_range[0], x_axis_range[-1], color = 'black', linestyles = '--')
        ax1_5.spines['bottom'].set_linewidth(1.6)
        ax1_5.spines['left'].set_linewidth(1.6)
        ax1_5.spines['right'].set_linewidth(1.6)
        ax1_5.spines['top'].set_linewidth(1.6)
        ax1_5.tick_params(axis = 'y', length=5, width = 1.6)
        ax1_5.tick_params(axis = 'x', length=5, width = 1.6)
        if i == len(window_l):
            ax1_5.set_ylabel(wd.split('-')[0], fontsize = 10, color = 'red')
        else:
            ax1_5.set_ylabel(wd.split('-')[0], fontsize = 10)
        #ax1_5.set_xticks(cord_list, x_ticks_l, fontsize = 0, rotation = 90)
        #ax1_5.set_xticks(cord_list, ['' for k in range(len(cord_list))], fontsize = 0, rotation = 90)
        ax1_5.set_xticks(cord_list)
        ax1_5.set_xticklabels(labels=['' for k in range(len(cord_list))], fontsize=0, rotation=90)

        cord_list_y = [0, 1]
        ax1_5.set_yticks(cord_list_y)
        ax1_5.set_yticklabels(labels=['', 1], fontsize=8)

        plt.ylim([0, 1])
        if len(target_site) != 0:
            site_use = []
            for i in range(len(target_site)):
                if target_site[i] < st:
                    pass
                elif target_site[i] < end:
                    site_use.append(target_site[i])
                else:
                    break
            plt.vlines(np.array(site_use) - st, 0, 1, color = 'black')
    
    ax5 = plt.subplot2grid((12, 7), (6 + i + 1 - 1, 0), rowspan=1,colspan=6,sharex=ax1)
    ax5.plot(list(bd_cell_score_original['bd_score'][st:ed]), color='black')
    ax5.bar(x_axis_range, list(bd_cell_score_original['bd_score'][st:ed]), color = '#D65F4D')
    ax5.spines['bottom'].set_linewidth(1.6)
    ax5.spines['left'].set_linewidth(1.6)
    ax5.spines['right'].set_linewidth(1.6)
    ax5.spines['top'].set_linewidth(1.6)
    ax5.tick_params(axis = 'y', length=5, width = 1.6)
    ax5.tick_params(axis = 'x', length=5, width = 1.6)
    ax5.set_ylabel('Bd score', fontsize = 10)
    #ax5.set_xticks(cord_list, ['' for k in range(len(cord_list))], fontsize = 0, rotation = 90)
    ax5.set_xticks(cord_list)
    ax5.set_xticklabels(labels=['' for k in range(len(cord_list))], fontsize=0, rotation=90)

    ax6 = plt.subplot2grid((12, 7), (6 + i + 2 - 1, 0), rowspan=1,colspan=6,sharex=ax1)
    ax6.plot(list(bd_cell_score_refined['bd_score'][st:ed]), color='black')
    ax6.bar(x_axis_range, list(bd_cell_score_refined['bd_score'][st:ed]), color = '#4392c3')
    ax6.spines['bottom'].set_linewidth(1.6)
    ax6.spines['left'].set_linewidth(1.6)
    ax6.spines['right'].set_linewidth(1.6)
    ax6.spines['top'].set_linewidth(1.6)
    ax6.tick_params(axis = 'y', length=5, width = 1.6)
    ax6.tick_params(axis = 'x', length=5, width = 1.6)
    ax6.set_ylabel('TSL \n Bd score', fontsize = 10, color = 'red')
    
    ax=plt.gca()
    ax.spines['bottom'].set_linewidth(1.8)
    ax.spines['left'].set_linewidth(1.8)
    ax.spines['right'].set_linewidth(1.8)
    ax.spines['top'].set_linewidth(1.8)
    ax.tick_params(axis = 'y', length=4, width = 1.8)
    ax.tick_params(axis = 'x', length=4, width = 1.8)
    #ax.set_xticks(cord_list, x_ticks_l, fontsize = 10, rotation = -30)
    ax.set_xticks(cord_list)
    ax.set_xticklabels(labels = x_ticks_l, fontsize = 10, rotation = -30)

def DrawBdScoreMultiScalePvalue_old(mat_dense, Chr, st, ed, bd_cell_score_original, bd_cell_score_refined, df_bd_pvalue_result, peak_list, resolution, save_name, p_cut, target_site = [], fgsize = (12, 8), bin_size = 10):
    plt.figure(figsize=(fgsize[0], fgsize[1]))
    x_axis_range = range(len(bd_cell_score_original['bd_score'][st:ed]))
    
    start_ =  st * resolution 
    end_ = ed * resolution

    cord_list = []
    pos_list = []
    pos_start = start_ 
    x_ticks_l = []
    y_ticks_l = []
    for i in range(ed - st):
        if i % bin_size == 0:
            cord_list.append(i)
            pos = pos_start + i*resolution
            pos_list.append(pos)
            if i + bin_size < ed - st:
                pos_label = str(pos / 1000000)
            else:
                #pos_label = str(pos / 1000000) + '(Mb)'
                pos_label = str(pos / 1000000) 
            x_ticks_l.append(pos_label)
            y_ticks_l.append(str(pos / 1000000))
    region_name = Chr + ':' + str(start_ / 1000000) + '-' + str(end_ / 1000000) + ' Mb'
         
    ax1 = plt.subplot2grid((12, 7), (0, 0), rowspan=6,colspan=6)
    start = int(start_ / resolution)
    end = int(end_ / resolution)
    dense_matrix_part = mat_dense[start:end, start:end]
    #plt.figure(figsize=(10, 10))
    #plt.figure(figsize=(4, 4))
    #ax = sns.heatmap(data = dense_matrix_part, square=True, yticklabels=False,  xticklabels=False, cmap='gist_gray_r', vmin = np.percentile(dense_matrix_part, 10), vmax = np.percentile(dense_matrix_part, 90))
    #img = ax1.imshow(dense_matrix_part, cmap='gist_gray_r', vmin = np.percentile(dense_matrix_part, 10), vmax = np.percentile(dense_matrix_part, 90))
    img = ax1.imshow(dense_matrix_part, cmap='coolwarm', vmin = np.percentile(dense_matrix_part, 10), vmax = np.percentile(dense_matrix_part, 90))
    
    ax1.set_xticks([])
    #ax1.set_yticks([])
    ax1.spines['bottom'].set_linewidth(0)
    ax1.spines['left'].set_linewidth(1.6)
    ax1.spines['right'].set_linewidth(0)
    ax1.spines['top'].set_linewidth(0)
    ax1.tick_params(axis = 'y', length=5, width = 1.6)
    ax1.tick_params(axis = 'x', length=5, width = 1.6)
    plt.xticks(cord_list, x_ticks_l, fontsize = 10)
    plt.yticks(cord_list, y_ticks_l, fontsize = 10)
    ax1.set_title('TAD landscape of region:' + region_name, fontsize=12, pad = 15.0)

    cax = plt.subplot2grid((12, 7), (0, 6), rowspan=6,colspan=1)
    #divider = make_axes_locatable(cax)
    #cax = divider.append_axes("right", size="1.5%", pad= 0.2)
    #cbar = plt.colorbar(img, cax=cax, ticks=MultipleLocator(2.0), format="%.1f",orientation='vertical',extendfrac='auto',spacing='uniform')
    cbaxes = inset_axes(cax, width="30%", height="100%", loc=3) 
    plt.colorbar(img, cax = cbaxes, orientation='vertical')
    cax.spines['bottom'].set_linewidth(0)
    cax.spines['left'].set_linewidth(0)
    cax.spines['right'].set_linewidth(0)
    cax.spines['top'].set_linewidth(0)
    cax.tick_params(axis = 'y', length=0, width = 0)
    cax.tick_params(axis = 'x', length=0, width = 0)
    cax.set_xticks([])
    cax.set_yticks([])

    ax1_5 = plt.subplot2grid((12, 7), (6, 0), rowspan=1,colspan=6,sharex=ax1)
    ax1_5.plot(list(df_bd_pvalue_result[peak_list[-1]][st:ed]), color='black')
    ax1_5.bar(x_axis_range, list(df_bd_pvalue_result[peak_list[-1]][st:ed]), label=peak_list[-1], color='#B4E0F9')
    if p_cut != 0:
        ax1_5.hlines(p_cut, x_axis_range[0], x_axis_range[-1], color = 'black', linestyles = '--')
    ax1_5.spines['bottom'].set_linewidth(1.6)
    ax1_5.spines['left'].set_linewidth(1.6)
    ax1_5.spines['right'].set_linewidth(1.6)
    ax1_5.spines['top'].set_linewidth(1.6)
    ax1_5.tick_params(axis = 'y', length=5, width = 1.6)
    ax1_5.tick_params(axis = 'x', length=5, width = 1.6)
    ax1_5.set_ylabel(peak_list[-1].split('-')[0], fontsize = 10)

    ax2 = plt.subplot2grid((12, 7), (7, 0), rowspan=1,colspan=6,sharex=ax1)
    ax2.plot(list(df_bd_pvalue_result[peak_list[-2]][st:ed]), color='black')
    ax2.bar(x_axis_range, list(df_bd_pvalue_result[peak_list[-2]][st:ed]), label=peak_list[-2], color='#B4E0F9')
    ax2.hlines(p_cut, x_axis_range[0], x_axis_range[-1], color = 'black', linestyles = '--')
    ax2.spines['bottom'].set_linewidth(1.6)
    ax2.spines['left'].set_linewidth(1.6)
    ax2.spines['right'].set_linewidth(1.6)
    ax2.spines['top'].set_linewidth(1.6)
    ax2.tick_params(axis = 'y', length=5, width = 1.6)
    ax2.tick_params(axis = 'x', length=5, width = 1.6)
    ax2.set_ylabel(peak_list[-2].split('-')[0], fontsize = 10)

    ax3 = plt.subplot2grid((12, 7), (8, 0), rowspan=1,colspan=6,sharex=ax1)
    ax3.plot(list(df_bd_pvalue_result[peak_list[-3]][st:ed]), color='black')
    ax3.bar(x_axis_range, list(df_bd_pvalue_result[peak_list[-3]][st:ed]), label=peak_list[-3], color='#B4E0F9')
    if p_cut != 0:
        ax3.hlines(p_cut, x_axis_range[0], x_axis_range[-1], color = 'black', linestyles = '--')
    ax3.spines['bottom'].set_linewidth(1.6)
    ax3.spines['left'].set_linewidth(1.6)
    ax3.spines['right'].set_linewidth(1.6)
    ax3.spines['top'].set_linewidth(1.6)
    ax3.tick_params(axis = 'y', length=5, width = 1.6)
    ax3.tick_params(axis = 'x', length=5, width = 1.6)
    ax3.set_ylabel(peak_list[-3].split('-')[0], fontsize =10)

    ax4 = plt.subplot2grid((12, 7), (9, 0), rowspan=1,colspan=6,sharex=ax1)
    ax4.plot(list(df_bd_pvalue_result[peak_list[-4]][st:ed]), color='black')
    ax4.bar(x_axis_range, list(df_bd_pvalue_result[peak_list[-4]][st:ed]), label=peak_list[-4], color='#B4E0F9')
    if p_cut != 0:
        ax4.hlines(p_cut, x_axis_range[0], x_axis_range[-1], color = 'black', linestyles = '--')
    ax4.spines['bottom'].set_linewidth(1.6)
    ax4.spines['left'].set_linewidth(1.6)
    ax4.spines['right'].set_linewidth(1.6)
    ax4.spines['top'].set_linewidth(1.6)
    ax4.tick_params(axis = 'y', length=5, width = 1.6)
    ax4.tick_params(axis = 'x', length=5, width = 1.6)
    ax4.set_ylabel(peak_list[-4].split('-')[0], fontsize = 10)

    ax5 = plt.subplot2grid((12, 7), (10, 0), rowspan=1,colspan=6,sharex=ax4)
    ax5.plot(list(bd_cell_score_original['bd_score'][st:ed]), color='black')
    ax5.bar(x_axis_range, list(bd_cell_score_original['bd_score'][st:ed]), color = '#D65F4D')
    ax5.spines['bottom'].set_linewidth(1.6)
    ax5.spines['left'].set_linewidth(1.6)
    ax5.spines['right'].set_linewidth(1.6)
    ax5.spines['top'].set_linewidth(1.6)
    ax5.tick_params(axis = 'y', length=5, width = 1.6)
    ax5.tick_params(axis = 'x', length=5, width = 1.6)
    ax5.set_ylabel('Bd score', fontsize = 10)
    
    ax6 = plt.subplot2grid((12, 7), (11, 0), rowspan=1,colspan=6,sharex=ax5)
    ax6.plot(list(bd_cell_score_refined['bd_score'][st:ed]), color='black')
    ax6.bar(x_axis_range, list(bd_cell_score_refined['bd_score'][st:ed]), color = '#4392c3')
    ax6.spines['bottom'].set_linewidth(1.6)
    ax6.spines['left'].set_linewidth(1.6)
    ax6.spines['right'].set_linewidth(1.6)
    ax6.spines['top'].set_linewidth(1.6)
    ax6.tick_params(axis = 'y', length=5, width = 1.6)
    ax6.tick_params(axis = 'x', length=5, width = 1.6)
    ax6.set_ylabel('Bd score', fontsize = 10)

    if len(target_site) != 0:
        site_use = []
        for i in range(len(target_site)):
            if target_site[i] < st:
                pass
            elif target_site[i] <= end:
                site_use.append(target_site[i])
            else:
                break
        plt.vlines(np.array(site_use) - st, 0, 14)
    if save_name != '':
        plt.savefig(save_name, format = 'svg') 
    plt.show()
    #fig = plt.gcf() #获取当前figure
    #plt.close(fig)



def draw_square_region(st, ed, color, size_v, size_h):  
    ## 画竖线
    plt.vlines(st, st, ed, colors=color, linestyles='solid', linewidths=size_v)
    plt.vlines(ed, st, ed, colors=color, linestyles='solid', linewidths=size_v)
    ## 画横线
    plt.hlines(st, st, ed, colors=color, linestyles='solid', linewidths=size_h)
    plt.hlines(ed, st, ed, colors=color, linestyles='solid', linewidths=size_h)


def draw_tad_region(st, ed, color, size_v, size_h):  
    ## 画竖线
    plt.vlines(st, st, ed, colors=color, linestyles='solid', linewidths=size_v)
    plt.vlines(ed, st, ed, colors=color, linestyles='solid', linewidths=size_v)
    ## 画横线
    plt.hlines(st, st, ed, colors=color, linestyles='solid', linewidths=size_h)
    plt.hlines(ed, st, ed, colors=color, linestyles='solid', linewidths=size_h)

 
def draw_tad_region_upper_half(st, ed, range_t, color, size_v, size_h):  
    if st < 0:
        plt.vlines(ed, 0, ed, colors=color, linestyles='solid', linewidths=size_v)
    elif ed > range_t:
        plt.hlines(st, st, range_t, colors=color, linestyles='solid', linewidths=size_v)
    else:## 画竖线
        #plt.vlines(st, st, ed, colors=color, linestyles='solid', linewidths=size_v)
        plt.vlines(ed, st, ed, colors=color, linestyles='solid', linewidths=size_v)
        ## 画横线
        plt.hlines(st, st, ed, colors=color, linestyles='solid', linewidths=size_h)
        #plt.hlines(ed, st, ed, colors=color, linestyles='solid', linewidths=size_h)

def draw_tad_region_lower_half(st, ed, range_t, color, size_v, size_h):  
    if st < 0:
        plt.hlines(ed, 0, ed, colors=color, linestyles='solid', linewidths=size_h)
    elif ed > range_t: 
        plt.vlines(st, st, range_t, colors=color, linestyles='solid', linewidths=size_v)
    else:
        ## 画竖线
        plt.vlines(st, st, ed, colors=color, linestyles='solid', linewidths=size_v)
        #plt.vlines(ed, st, ed, colors=color, linestyles='solid', linewidths=size_v)
        ## 画横线
        #plt.hlines(st, st, ed, colors=color, linestyles='solid', linewidths=size_h)
        plt.hlines(ed, st, ed, colors=color, linestyles='solid', linewidths=size_h)

def get_bd_type_symbol(df_bd_region_type, bd_score_cell_combine, symbol_dic):
    bd_symbol = np.zeros(len(bd_score_cell_combine))
    for i in range(len(df_bd_region_type)):
        region = df_bd_region_type['region'][i]
        bd_type = df_bd_region_type['region_type_adjust'][i]
        symbol = symbol_dic[bd_type]
        for x in region:
            bd_symbol[x] = symbol
    return bd_symbol


def DrawConsTADsAndBdRegion(contact_map, Chr, st, ed, TAD_list, bd_score_cell_combine, bd_symbol, resolution, fgsize = (10, 10), save_name = '', bin_size = 10):
    x_axis_range = range(len(bd_score_cell_combine['bd_score'][st:ed]))
    start = st * resolution
    end = ed * resolution + resolution
    start_ = start / 1000000 
    end_ = end / 1000000
    region_name = Chr + ':' + str(start_) + '-' + str(end_) + ' Mb'
    x_ticks_l = []
    y_ticks_l = []
    cord_list = []
    for i in range(ed - st):
        if i % bin_size == 0:
            pos = (st+i) * resolution
            cord_list.append(i)
            x_ticks_l.append(str(pos / 1000000))
            y_ticks_l.append(str(pos / 1000000))
    
    plt.figure(figsize=(fgsize[0], fgsize[1]))     
    ax0 = plt.subplot2grid((8, 8), (0, 0), rowspan=6,colspan=6)
    dense_matrix_part = contact_map[st:ed+1, st:ed+1]
    #img = ax0.imshow(dense_matrix_part, cmap='seismic', vmin = np.percentile(dense_matrix_part, 10), vmax = np.percentile(dense_matrix_part, 91))
    img = ax0.imshow(dense_matrix_part, cmap='coolwarm', vmin = np.percentile(dense_matrix_part, 10), vmax = np.percentile(dense_matrix_part, 90))
    ax0.set_xticks([])
    #ax0.set_yticks([])
    ax0.spines['bottom'].set_linewidth(0)
    ax0.spines['left'].set_linewidth(1.6)
    ax0.spines['right'].set_linewidth(0)
    ax0.spines['top'].set_linewidth(0)
    ax0.tick_params(axis = 'y', length=5, width = 1.6)
    ax0.tick_params(axis = 'x', length=5, width = 1.6)
    plt.xticks(cord_list, x_ticks_l, fontsize = 10)
    plt.yticks(cord_list, y_ticks_l, fontsize = 10)
    ax0.set_title(region_name, fontsize=12, pad = 15.0)
    
    TAD_color = 'black'
    if len(TAD_list) != 0:
        for TAD in TAD_list:
            st_tad = TAD[0] - st
            ed_tad = TAD[1] - st
            #print(st_tad, ed_tad)
            draw_tad_region(st_tad, ed_tad, TAD_color, size_v=3, size_h=3)
            #draw_tad_region_upper_half(st_tad, ed_tad, TAD_color, size_v=3, size_h=3)

    cax = plt.subplot2grid((8, 8), (0, 6), rowspan=6,colspan=1)
    #divider = make_axes_locatable(cax)
    #cax = divider.append_axes("right", size="1.5%", pad= 0.2)
    #cbar = plt.colorbar(img, cax=cax, ticks=MultipleLocator(2.0), format="%.1f",orientation='vertical',extendfrac='auto',spacing='uniform')
    cbaxes = inset_axes(cax, width="30%", height="94%", loc=3) 
    plt.colorbar(img, cax = cbaxes, orientation='vertical')
    cax.spines['bottom'].set_linewidth(0)
    cax.spines['left'].set_linewidth(0)
    cax.spines['right'].set_linewidth(0)
    cax.spines['top'].set_linewidth(0)
    cax.tick_params(axis = 'y', length=0, width = 0)
    cax.tick_params(axis = 'x', length=0, width = 0)
    cax.set_xticks([])
    cax.set_yticks([])

    ax1 = plt.subplot2grid((8, 8), (6, 0), rowspan=1, colspan=6, sharex=ax0)    
    ax1.plot(x_axis_range, bd_score_cell_combine['bd_score'][st:ed], marker = '.', linewidth = 2, c = 'black')
    ax1.bar(x_axis_range, list(bd_score_cell_combine['bd_score'][st:ed]))
    plt.ylabel('Bd score')
    ax1.set_xticks([])
    #ax1.set_yticks([])
    ax1.spines['bottom'].set_linewidth(1.6)
    ax1.spines['left'].set_linewidth(1.6)
    ax1.spines['right'].set_linewidth(1.6)
    ax1.spines['top'].set_linewidth(1.6)
    ax1.tick_params(axis = 'y', length=5, width = 1.6)
    ax1.tick_params(axis = 'x', length=5, width = 1.6)

    ax2 = plt.subplot2grid((8, 8), (7, 0), rowspan=1,colspan=6, sharex=ax0)    
    bd_data = []
    cmap=['#B0B0B0','#459457','#D65F4D','#4392C3']
    my_cmap = ListedColormap(cmap)
    bounds=[0,0.9,1.9,2.9,3.9]
    norm = matplotlib.colors.BoundaryNorm(bounds, my_cmap.N)    
    for i in range(10):
        bd_data.append(bd_symbol[st:ed])
    #bd_data_expand = np.reshape(np.array(bd_data), (10, len(bd_data[0])))
    ax2.imshow(bd_data, cmap = my_cmap, norm = norm)
    plt.ylabel('Bd type')
    plt.yticks([1, 10], ['', ''])
    ax2.spines['bottom'].set_linewidth(1.6)
    ax2.spines['left'].set_linewidth(1.6)
    ax2.spines['right'].set_linewidth(1.6)
    ax2.spines['top'].set_linewidth(1.6)
    ax2.tick_params(axis = 'y', length=0, width = 0)
    ax2.tick_params(axis = 'x', length=0, width = 0)    
     
    if save_name != '':
        plt.savefig(save_name, format = 'svg', transparent = True) 
    plt.show()
    #fig = plt.gcf() #获取当前figure
    #plt.close(fig)


def draw_bd_region_case_plus(st, ed, contact_map, TAD_list, bd_score_cell_combine, bd_symbol, domain_symbol, repli_signal, Chr, save_name = '', bin_size = 8, resolution = 50000):
    x_axis_range = range(len(bd_score_cell_combine['bd_score'][st:ed]))
    start = st * resolution
    end = ed * resolution + resolution
    start_ = start / 1000000 
    end_ = end / 1000000
    region_name = Chr + ':' + str(start_) + '-' + str(end_) + ' Mb'
    x_ticks_l = []
    y_ticks_l = []
    cord_list = []
    for i in range(ed - st):
        if i % bin_size == 0:
            pos = (st+i) * resolution
            cord_list.append(i)
            x_ticks_l.append(str(pos / 1000000))
            y_ticks_l.append(str(pos / 1000000))
    
    plt.figure(figsize=(8, 8))     
    ax0 = plt.subplot2grid((8, 8), (0, 0), rowspan=4,colspan=4)
    dense_matrix_part = contact_map[st:ed+1, st:ed+1]
    img = ax0.imshow(dense_matrix_part, cmap='seismic', vmin = np.percentile(dense_matrix_part, 10), vmax = np.percentile(dense_matrix_part, 91))
    #img = ax0.imshow(dense_matrix_part, cmap='coolwarm', vmin = np.percentile(dense_matrix_part, 10), vmax = np.percentile(dense_matrix_part, 90))
    ax0.set_xticks([])
    #ax0.set_yticks([])
    ax0.spines['bottom'].set_linewidth(0)
    ax0.spines['left'].set_linewidth(1.6)
    ax0.spines['right'].set_linewidth(0)
    ax0.spines['top'].set_linewidth(0)
    ax0.tick_params(axis = 'y', length=5, width = 1.6)
    ax0.tick_params(axis = 'x', length=5, width = 1.6)
    plt.xticks(cord_list, x_ticks_l, FontSize = 10)
    plt.yticks(cord_list, y_ticks_l, FontSize = 10)
    ax0.set_title(region_name, fontsize=12, pad = 15.0)
    
    TAD_color = 'black'
    if len(TAD_list) != 0:
        for TAD in TAD_list:
            st_tad = TAD[0] - st
            ed_tad = TAD[1] - st
            print(st_tad, ed_tad)
            draw_tad_region(st_tad, ed_tad, TAD_color, size_v=3, size_h=3)
            #draw_tad_region_upper_half(st_tad, ed_tad, TAD_color, size_v=3, size_h=3)

    cax = plt.subplot2grid((8, 8), (0, 4), rowspan=4,colspan=1)
    #divider = make_axes_locatable(cax)
    #cax = divider.append_axes("right", size="1.5%", pad= 0.2)
    #cbar = plt.colorbar(img, cax=cax, ticks=MultipleLocator(2.0), format="%.1f",orientation='vertical',extendfrac='auto',spacing='uniform')
    cbaxes = inset_axes(cax, width="30%", height="50%", loc=2) 
    plt.colorbar(img, cax = cbaxes, orientation='vertical')
    cax.spines['bottom'].set_linewidth(0)
    cax.spines['left'].set_linewidth(0)
    cax.spines['right'].set_linewidth(0)
    cax.spines['top'].set_linewidth(0)
    cax.tick_params(axis = 'y', length=0, width = 0)
    cax.tick_params(axis = 'x', length=0, width = 0)
    cax.set_xticks([])
    cax.set_yticks([])

    ax1 = plt.subplot2grid((8, 8), (4, 0), rowspan=1, colspan=4, sharex=ax0)    
    ax1.plot(x_axis_range, bd_score_cell_combine['bd_score'][st:ed], marker = '.', linewidth = 2, c = 'black')
    ax1.bar(x_axis_range, list(bd_score_cell_combine['bd_score'][st:ed]))
    plt.ylabel('bd_score')
    ax1.set_xticks([])
    #ax1.set_yticks([])
    ax1.spines['bottom'].set_linewidth(1.6)
    ax1.spines['left'].set_linewidth(1.6)
    ax1.spines['right'].set_linewidth(1.6)
    ax1.spines['top'].set_linewidth(1.6)
    ax1.tick_params(axis = 'y', length=5, width = 1.6)
    ax1.tick_params(axis = 'x', length=5, width = 1.6)

    ax2 = plt.subplot2grid((8, 8), (5, 0), rowspan=1,colspan=4, sharex=ax0)    
    bd_data = []
    cmap=['#B0B0B0','#459457','#D65F4D','#4392C3']
    my_cmap = ListedColormap(cmap)
    bounds=[0,0.9,1.9,2.9,3.9]
    norm = matplotlib.colors.BoundaryNorm(bounds, my_cmap.N)    
    for i in range(10):
        bd_data.append(bd_symbol[st:ed])
    #bd_data_expand = np.reshape(np.array(bd_data), (10, len(bd_data[0])))
    ax2.imshow(bd_data, cmap = my_cmap, Norm = norm)
    plt.ylabel('bd type')
    ax2.spines['bottom'].set_linewidth(1.6)
    ax2.spines['left'].set_linewidth(1.6)
    ax2.spines['right'].set_linewidth(1.6)
    ax2.spines['top'].set_linewidth(1.6)
    ax2.tick_params(axis = 'y', length=0, width = 0)
    ax2.tick_params(axis = 'x', length=0, width = 0)    
    
    ax3 = plt.subplot2grid((8, 8), (6, 0), rowspan=1,colspan=4, sharex=ax0)    
    domain_data = []
    cmap = ['#B0B0B0', '#3A4CC0', '#7C9FF9', '#EDCBBA', '#E9795E', '#BC1F2C']
    my_cmap = ListedColormap(cmap)
    bounds=[0, 0.9, 1.9, 2.9, 3.9, 4.9, 5.9]
    norm = matplotlib.colors.BoundaryNorm(bounds, my_cmap.N)    
    for i in range(10):
        domain_data.append(domain_symbol[st:ed])
    #bd_data_expand = np.reshape(np.array(bd_data), (10, len(bd_data[0])))
    ax3.imshow(domain_data, cmap = my_cmap, Norm = norm)
    plt.ylabel('Domain type')
    ax3.spines['bottom'].set_linewidth(1.6)
    ax3.spines['left'].set_linewidth(1.6)
    ax3.spines['right'].set_linewidth(1.6)
    ax3.spines['top'].set_linewidth(1.6)
    ax3.tick_params(axis = 'y', length=0, width = 0)
    ax3.tick_params(axis = 'x', length=0, width = 0)    

    ax4 = plt.subplot2grid((8, 8), (7, 0), rowspan=1, colspan=4, sharex=ax0)    
    ax4.plot(x_axis_range, repli_signal[st:ed], marker = '.', linewidth = 2, c = 'black')
    ax4.bar(x_axis_range, list(repli_signal[st:ed]))
    plt.ylabel('Replication timing')
    ax4.set_xticks([])
    #ax1.set_yticks([])
    ax4.spines['bottom'].set_linewidth(1.6)
    ax4.spines['left'].set_linewidth(1.6)
    ax4.spines['right'].set_linewidth(1.6)
    ax4.spines['top'].set_linewidth(1.6)
    ax4.tick_params(axis = 'y', length=5, width = 1.6)
    ax4.tick_params(axis = 'x', length=5, width = 1.6)
         
    if save_name != '':
        plt.savefig(save_name, format = 'svg', transparent = True) 
    plt.show()
    #fig = plt.gcf() #获取当前figure
    #plt.close(fig)


def get_tad_list_in_target_ranges(st, ed, df_tad_use_1, pos_type = 'bin', resolution = 50000):
    if pos_type == 'cord':
        df_tad_use = copy.deepcopy(df_tad_use_1)
        df_tad_use['start'] = np.array(df_tad_use['start'] / resolution).astype(np.int32)
        df_tad_use['end'] = np.array(df_tad_use['end'] / resolution).astype(np.int32) - 1        
    else:
        df_tad_use = copy.deepcopy(df_tad_use_1)
    TAD_list = []
    for i in range(len(df_tad_use)):
        start = df_tad_use['start'][i]
        end = df_tad_use['end'][i]
        if start > st and end < ed:
            st_bin = start
            ed_bin = end
            TAD_list.append((st_bin, ed_bin))
        elif start < st and ( st < end <= ed):
            st_bin = start
            ed_bin = end
            TAD_list.append((st_bin, ed_bin))
        elif (ed > start >= st) and end > ed:
            st_bin = start
            ed_bin = end
            TAD_list.append((st_bin, ed_bin))
    return TAD_list


def matrix_part_max_norm(mat_region):
    vec_diag = np.diag(mat_region) 
    mat_diag = np.diag(vec_diag)
    mat_region -= mat_diag
    mat_region = mat_region / np.max(mat_region)
    return mat_region

def DrawPairWiseMapCompare(cell_1, cell_2, mat_dense1, mat_dense2, Chr, st, ed, bd_cell_score1, bd_cell_score2, TAD_list_1, TAD_list_2, resolution, figsize = (6,8), TAD_color_1 = 'black', TAD_color_2 = 'white', target_site = [], save_name = '', bin_size = 10): 
    x_axis_range = range(len(bd_cell_score1['bd_score'][st:ed]))
    start_ =  st * resolution 
    end_ = ed * resolution
    start = int(start_ / resolution)
    end = int(end_ / resolution)
    cord_list = []
    pos_list = []
    pos_start = start_ 
    x_ticks_l = []
    y_ticks_l = []
    for i in range(ed - st):
        if i % bin_size == 0:
            cord_list.append(i)
            pos = pos_start + i*resolution
            pos_list.append(pos)
            if i + bin_size < ed - st:
                pos_label = str(pos / 1000000)
            else:
                #pos_label = str(pos / 1000000) + '(Mb)'
                pos_label = str(pos / 1000000) 
            x_ticks_l.append(pos_label)
            y_ticks_l.append(str(pos / 1000000))
    region_name = Chr + ':' + str(start_ / 1000000) + '-' + str(end_ / 1000000) + ' Mb'
    
    dense_matrix_part1 = copy.deepcopy(mat_dense1[start:end, start:end])
    dense_matrix_norm1 = matrix_part_max_norm(dense_matrix_part1)
    dense_matrix_part2 = copy.deepcopy(mat_dense2[start:end, start:end])
    dense_matrix_norm2 = matrix_part_max_norm(dense_matrix_part2)
    
    dense_matrix_combine = np.triu(dense_matrix_norm1) + np.tril(-dense_matrix_norm2)
    #vmax = np.percentile(dense_matrix_combine, 90)
    #vmin = -vmax     
    vmax = np.percentile(dense_matrix_norm1, 90)
    vmin = -np.percentile(dense_matrix_norm2, 90)
    #norm = mcolors.DivergingNorm(vmin=vmin, vcenter=0, vmax=vmax)
    norm = mcolors.TwoSlopeNorm(vmin=vmin, vcenter=0, vmax=vmax)
    #plt.figure(figsize=(6,6))
    #plt.imshow(dense_matrix_combine, cmap = 'seismic', vmin = vmin, vmax =vmax, norm=norm) 
    #plt.imshow(dense_matrix_combine, cmap = 'coolwarm', vmin = vmin, vmax =vmax, norm=norm) 
    #plt.colorbar()

    plt.figure(figsize=(figsize[0], figsize[1]))
    ax1 = plt.subplot2grid((9, 7), (0, 0), rowspan=6,colspan=6)
    #img = ax1.imshow(dense_matrix_combine, cmap = 'coolwarm', vmin = vmin, vmax =vmax, norm=norm) 
    img = ax1.imshow(dense_matrix_combine, cmap = 'coolwarm', norm=norm) 
    ax1.set_xticks([])
    #ax1.set_yticks([])
    ax1.spines['bottom'].set_linewidth(0)
    ax1.spines['left'].set_linewidth(1.6)
    ax1.spines['right'].set_linewidth(0)
    ax1.spines['top'].set_linewidth(0)
    ax1.tick_params(axis = 'y', length=5, width = 1.6)
    ax1.tick_params(axis = 'x', length=5, width = 1.6)
    #ax1.set_xticks(cord_list, ['' for k in range(len(cord_list))], fontsize = 0, rotation = 90)
    ax1.set_xticks(cord_list)
    ax1.set_xticklabels(labels=['' for k in range(len(cord_list))], fontsize=0, rotation=90)

    plt.yticks(cord_list, y_ticks_l, fontsize = 10)
    ax1.set_title(cell_1 + ' vs ' + cell_2 + ' in ' + region_name, fontsize=12, pad = 15.0)

    range_t = ed - st - 1
    if len(TAD_list_1) != 0:
        for i in range(len(TAD_list_1)):
            TAD = TAD_list_1[i]
            st_tad = TAD[0] - st
            ed_tad = TAD[1] - st 
            #print(st_tad, ed_tad)
            draw_tad_region_upper_half(st_tad, ed_tad, range_t, TAD_color_1, size_v=5, size_h=5)
    if len(TAD_list_2) != 0:
        for i in range(len(TAD_list_2)):
            TAD = TAD_list_2[i]
            st_tad = TAD[0] - st
            ed_tad = TAD[1] - st 
            #print(st_tad, ed_tad)
            draw_tad_region_lower_half(st_tad, ed_tad, range_t, TAD_color_2, size_v=5, size_h=5)
            
    cax = plt.subplot2grid((9, 7), (0, 6), rowspan=6,colspan=1)
    #divider = make_axes_locatable(cax)
    #cax = divider.append_axes("right", size="1.5%", pad= 0.2)
    #cbar = plt.colorbar(img, cax=cax, ticks=MultipleLocator(2.0), format="%.1f",orientation='vertical',extendfrac='auto',spacing='uniform')
    cbaxes = inset_axes(cax, width="30%", height="70%", loc=5) 
    plt.colorbar(img, cax = cbaxes, orientation='vertical')
    cax.spines['bottom'].set_linewidth(0)
    cax.spines['left'].set_linewidth(0)
    cax.spines['right'].set_linewidth(0)
    cax.spines['top'].set_linewidth(0)
    cax.tick_params(axis = 'y', length=0, width = 0)
    cax.tick_params(axis = 'x', length=0, width = 0)
    cax.set_xticks([])
    cax.set_yticks([])

    ax2 = plt.subplot2grid((9, 7), (6, 0), rowspan=1,colspan=6,sharex=ax1)
    ax2.plot(list(bd_cell_score1['bd_score'][st:ed]), color='black')
    ax2.bar(x_axis_range, list(bd_cell_score1['bd_score'][st:ed]), label='score1', color='#B70D28')
    ax2.spines['bottom'].set_linewidth(1.6)
    ax2.spines['left'].set_linewidth(1.6)
    ax2.spines['right'].set_linewidth(1.6)
    ax2.spines['top'].set_linewidth(1.6)
    ax2.tick_params(axis = 'y', length=5, width = 1.6)
    ax2.tick_params(axis = 'x', length=5, width = 1.6)
    #ax2.set_xticks(cord_list, ['' for k in range(len(cord_list))], fontsize = 0, rotation = 90)
    ax2.set_xticks(cord_list)
    ax2.set_xticklabels(labels=['' for k in range(len(cord_list))], fontsize=0, rotation=90)

    ax2.set_ylabel(cell_1 + ' \n TSL', fontsize = 10)

    ax3 = plt.subplot2grid((9, 7), (7, 0), rowspan=1,colspan=6,sharex=ax1)
    ax3.plot(list(bd_cell_score2['bd_score'][st:ed]), color='black')
    ax3.bar(x_axis_range, list(bd_cell_score2['bd_score'][st:ed]), label='score2', color='#3D50C3')
    ax3.spines['bottom'].set_linewidth(1.6)
    ax3.spines['left'].set_linewidth(1.6)
    ax3.spines['right'].set_linewidth(1.6)
    ax3.spines['top'].set_linewidth(1.6)
    ax3.tick_params(axis = 'y', length=5, width = 1.6)
    ax3.tick_params(axis = 'x', length=5, width = 1.6)
    ax3.set_ylabel(cell_2 + ' \n TSL', fontsize = 10)
    if len(target_site) != 0:
        site_use = []
        for i in range(len(target_site)):
            if target_site[i] < st:
                pass
            elif target_site[i] <= end:
                site_use.append(target_site[i])
            else:
                break
        plt.vlines(np.array(site_use) - st, 0, 14, linestyles='--')
    
    ax=plt.gca()
    ax.spines['bottom'].set_linewidth(1.8)
    ax.spines['left'].set_linewidth(1.8)
    ax.spines['right'].set_linewidth(1.8)
    ax.spines['top'].set_linewidth(1.8)
    ax.tick_params(axis = 'y', length=4, width = 1.8)
    ax.tick_params(axis = 'x', length=4, width = 1.8)
    #ax.set_xticks(cord_list, x_ticks_l, fontsize = 10, rotation = -30)
    ax.set_xticks(cord_list)
    ax.set_xticklabels(labels=x_ticks_l, fontsize=10, rotation=-0)

    if save_name != '':
        plt.savefig(save_name, format = 'svg') 
    plt.show()
    #fig = plt.gcf() #获取当前figure
    #plt.close(fig)



























